@@ -5,37 +5,35 @@ const common = require('../common.js');
 const bench = common.createBenchmark(main, {
   method: ['for', 'for-of', 'for-in', 'forEach'],
   count: [5, 10, 20, 100],
-  n: [5e6]
+  n: [5e6],
 });
 
 function useFor(n, items, count) {
   bench.start();
-  for (var i = 0; i < n; i++) {
-    for (var j = 0; j < count; j++) {
-      /* eslint-disable no-unused-vars */
+  for (let i = 0; i < n; i++) {
+    for (let j = 0; j < count; j++) {
+      // eslint-disable-next-line no-unused-vars
       const item = items[j];
-      /* esline-enable no-unused-vars */
     }
   }
   bench.end(n);
 }
 
 function useForOf(n, items) {
-  var item;
   bench.start();
-  for (var i = 0; i < n; i++) {
-    for (item of items) {}
+  for (let i = 0; i < n; i++) {
+    // eslint-disable-next-line no-unused-vars, no-empty
+    for (const item of items) {}
   }
   bench.end(n);
 }
 
 function useForIn(n, items) {
   bench.start();
-  for (var i = 0; i < n; i++) {
-    for (var j in items) {
-      /* eslint-disable no-unused-vars */
+  for (let i = 0; i < n; i++) {
+    for (const j in items) {
+      // eslint-disable-next-line no-unused-vars
       const item = items[j];
-      /* esline-enable no-unused-vars */
     }
   }
   bench.end(n);
@@ -43,7 +41,7 @@ function useForIn(n, items) {
 
 function useForEach(n, items) {
   bench.start();
-  for (var i = 0; i < n; i++) {
+  for (let i = 0; i < n; i++) {
     items.forEach((item) => {});
   }
   bench.end(n);
@@ -51,13 +49,11 @@ function useForEach(n, items) {
 
 function main({ n, count, method }) {
   const items = new Array(count);
-  var fn;
-  for (var i = 0; i < count; i++)
+  let fn;
+  for (let i = 0; i < count; i++)
     items[i] = i;
 
   switch (method) {
-    case '':
-      // Empty string falls through to next line as default, mostly for tests.
     case 'for':
       fn = useFor;
       break;
