@@ -7,6 +7,7 @@
 #include "src/compiler/access-builder.h"
 #include "src/compiler/common-operator.h"
 #include "src/compiler/js-graph.h"
+#include "src/compiler/js-heap-broker.h"
 #include "src/compiler/node-properties.h"
 #include "src/heap/factory.h"
 #include "src/objects/objects-inl.h"
@@ -270,7 +271,7 @@ bool MayAlias(MaybeHandle<Name> x, MaybeHandle<Name> y) {
 
 class LoadElimination::AliasStateInfo {
  public:
-  AliasStateInfo(const AbstractState* state, Node* object, Handle<Map> map)
+  AliasStateInfo(const AbstractState* state, Node* object, MapRef map)
       : state_(state), object_(object), map_(map) {}
   AliasStateInfo(const AbstractState* state, Node* object)
       : state_(state), object_(object) {}
@@ -280,7 +281,7 @@ class LoadElimination::AliasStateInfo {
  private:
   const AbstractState* state_;
   Node* object_;
-  MaybeHandle<Map> map_;
+  OptionalMapRef map_;
 };
 
 LoadElimination::AbstractField const* LoadElimination::AbstractField::KillConst(
@@ -336,15 +337,15 @@ void LoadElimination::AbstractField::Print() const {
 LoadElimination::AbstractMaps::AbstractMaps(Zone* zone)
     : info_for_node_(zone) {}
 
-LoadElimination::AbstractMaps::AbstractMaps(Node* object,
-                                            ZoneHandleSet<Map> maps, Zone* zone)
+LoadElimination::AbstractMaps::AbstractMaps(Node* object, ZoneRefSet<Map> maps,
+                                            Zone* zone)
     : info_for_node_(zone) {
   object = ResolveRenames(object);
   info_for_node_.insert(std::make_pair(object, maps));
 }
 
-bool LoadElimination::AbstractMaps::Lookup(
-    Node* object, ZoneHandleSet<Map>* object_maps) const {
+bool LoadElimination::AbstractMaps::Lookup(Node* object,
+                                           ZoneRefSet<Map>* object_maps) const {
   auto it = info_for_node_.find(ResolveRenames(object));
   if (it == info_for_node_.end()) return false;
   *object_maps = it->second;
@@ -372,7 +373,7 @@ LoadElimination::AbstractMaps const* LoadElimination::AbstractMaps::Merge(
   AbstractMaps* copy = zone->New<AbstractMaps>(zone);
   for (auto this_it : this->info_for_node_) {
     Node* this_object = this_it.first;
-    ZoneHandleSet<Map> this_maps = this_it.second;
+    ZoneRefSet<Map> this_maps = this_it.second;
     auto that_it = that->info_for_node_.find(this_object);
     if (that_it != that->info_for_node_.end() && that_it->second == this_maps) {
       copy->info_for_node_.insert(this_it);
@@ -382,7 +383,7 @@ LoadElimination::AbstractMaps const* LoadElimination::AbstractMaps::Merge(
 }
 
 LoadElimination::AbstractMaps const* LoadElimination::AbstractMaps::Extend(
-    Node* object, ZoneHandleSet<Map> maps, Zone* zone) const {
+    Node* object, ZoneRefSet<Map> maps, Zone* zone) const {
   AbstractMaps* that = zone->New<AbstractMaps>(*this);
   if (that->info_for_node_.size() >= kMaxTrackedObjects) {
     // We are tracking too many objects, which leads to bad performance.
@@ -400,9 +401,9 @@ void LoadElimination::AbstractMaps::Print() const {
   for (auto pair : info_for_node_) {
     os << "    #" << pair.first->id() << ":" << pair.first->op()->mnemonic()
        << std::endl;
-    ZoneHandleSet<Map> const& maps = pair.second;
+    ZoneRefSet<Map> const& maps = pair.second;
     for (size_t i = 0; i < maps.size(); ++i) {
-      os << "     - " << Brief(*maps[i]) << std::endl;
+      os << "     - " << Brief(*maps[i].object()) << std::endl;
     }
   }
 }
@@ -451,7 +452,7 @@ void LoadElimination::AbstractState::FieldsMerge(
     AbstractField const*& this_field = (*this_fields)[i];
     if (this_field) {
       if (that_fields[i]) {
-        this_field = this_field->Merge(that_fields[i], zone);
+        this_field = this_field->Merge(that_fields[i], zone, &fields_count_);
       } else {
         this_field = nullptr;
       }
@@ -469,8 +470,10 @@ void LoadElimination::AbstractState::Merge(AbstractState const* that,
   }
 
   // Merge the information we have about the fields.
-  FieldsMerge(&this->fields_, that->fields_, zone);
+  fields_count_ = 0;
   FieldsMerge(&this->const_fields_, that->const_fields_, zone);
+  const_fields_count_ = fields_count_;
+  FieldsMerge(&this->fields_, that->fields_, zone);
 
   // Merge the information we have about the maps.
   if (this->maps_) {
@@ -479,12 +482,12 @@ void LoadElimination::AbstractState::Merge(AbstractState const* that,
 }
 
 bool LoadElimination::AbstractState::LookupMaps(
-    Node* object, ZoneHandleSet<Map>* object_map) const {
+    Node* object, ZoneRefSet<Map>* object_map) const {
   return this->maps_ && this->maps_->Lookup(object, object_map);
 }
 
 LoadElimination::AbstractState const* LoadElimination::AbstractState::SetMaps(
-    Node* object, ZoneHandleSet<Map> maps, Zone* zone) const {
+    Node* object, ZoneRefSet<Map> maps, Zone* zone) const {
   AbstractState* that = zone->New<AbstractState>(*this);
   if (that->maps_) {
     that->maps_ = that->maps_->Extend(object, maps, zone);
@@ -556,14 +559,19 @@ LoadElimination::AbstractState const* LoadElimination::AbstractState::AddField(
     Node* object, IndexRange index_range, LoadElimination::FieldInfo info,
     Zone* zone) const {
   AbstractState* that = zone->New<AbstractState>(*this);
-  AbstractFields& fields =
-      info.const_field_info.IsConst() ? that->const_fields_ : that->fields_;
+  bool is_const = info.const_field_info.IsConst();
+  AbstractFields& fields = is_const ? that->const_fields_ : that->fields_;
   for (int index : index_range) {
+    int count_before = fields[index] ? fields[index]->count() : 0;
     if (fields[index]) {
-      fields[index] = fields[index]->Extend(object, info, zone);
+      fields[index] =
+          fields[index]->Extend(object, info, zone, that->fields_count_);
     } else {
       fields[index] = zone->New<AbstractField>(object, info, zone);
     }
+    int added = fields[index]->count() - count_before;
+    if (is_const) that->const_fields_count_ += added;
+    that->fields_count_ += added;
   }
   return that;
 }
@@ -580,6 +588,10 @@ LoadElimination::AbstractState::KillConstField(Node* object,
       if (this->const_fields_[index] != this_field) {
         if (!that) that = zone->New<AbstractState>(*this);
         that->const_fields_[index] = this_field;
+        int removed = this->const_fields_[index]->count() -
+                      that->const_fields_[index]->count();
+        that->const_fields_count_ -= removed;
+        that->fields_count_ -= removed;
       }
     }
   }
@@ -603,6 +615,9 @@ LoadElimination::AbstractState const* LoadElimination::AbstractState::KillField(
       if (this->fields_[index] != this_field) {
         if (!that) that = zone->New<AbstractState>(*this);
         that->fields_[index] = this_field;
+        int removed =
+            this->fields_[index]->count() - that->fields_[index]->count();
+        that->fields_count_ -= removed;
       }
     }
   }
@@ -614,7 +629,9 @@ LoadElimination::AbstractState::KillFields(Node* object, MaybeHandle<Name> name,
                                            Zone* zone) const {
   AliasStateInfo alias_info(this, object);
   for (size_t i = 0;; ++i) {
-    if (i == fields_.size()) return this;
+    if (i == fields_.size()) {
+      return this;
+    }
     if (AbstractField const* this_field = this->fields_[i]) {
       AbstractField const* that_field =
           this_field->Kill(alias_info, name, zone);
@@ -624,6 +641,8 @@ LoadElimination::AbstractState::KillFields(Node* object, MaybeHandle<Name> name,
         while (++i < fields_.size()) {
           if (this->fields_[i] != nullptr) {
             that->fields_[i] = this->fields_[i]->Kill(alias_info, name, zone);
+            int removed = this->fields_[i]->count() - that->fields_[i]->count();
+            that->fields_count_ -= removed;
           }
         }
         return that;
@@ -639,6 +658,8 @@ LoadElimination::AbstractState const* LoadElimination::AbstractState::KillAll(
     if (const_fields_[i]) {
       AbstractState* that = zone->New<AbstractState>();
       that->const_fields_ = const_fields_;
+      that->const_fields_count_ = const_fields_count_;
+      that->fields_count_ = const_fields_count_;
       return that;
     }
   }
@@ -690,11 +711,11 @@ bool LoadElimination::AliasStateInfo::MayAlias(Node* other) const {
     return false;
   }
   // Decide aliasing based on maps (if available).
-  Handle<Map> map;
-  if (map_.ToHandle(&map)) {
-    ZoneHandleSet<Map> other_maps;
+  if (map_.has_value()) {
+    MapRef map = *map_;
+    ZoneRefSet<Map> other_maps;
     if (state_->LookupMaps(other, &other_maps) && other_maps.size() == 1) {
-      if (map.address() != other_maps.at(0).address()) {
+      if (map != other_maps.at(0)) {
         return false;
       }
     }
@@ -740,12 +761,12 @@ void LoadElimination::AbstractStateForEffectNodes::Set(
 }
 
 Reduction LoadElimination::ReduceMapGuard(Node* node) {
-  ZoneHandleSet<Map> const& maps = MapGuardMapsOf(node->op());
+  ZoneRefSet<Map> const& maps = MapGuardMapsOf(node->op());
   Node* const object = NodeProperties::GetValueInput(node, 0);
   Node* const effect = NodeProperties::GetEffectInput(node);
   AbstractState const* state = node_states_.Get(effect);
   if (state == nullptr) return NoChange();
-  ZoneHandleSet<Map> object_maps;
+  ZoneRefSet<Map> object_maps;
   if (state->LookupMaps(object, &object_maps)) {
     if (maps.contains(object_maps)) return Replace(effect);
     // TODO(turbofan): Compute the intersection.
@@ -755,12 +776,12 @@ Reduction LoadElimination::ReduceMapGuard(Node* node) {
 }
 
 Reduction LoadElimination::ReduceCheckMaps(Node* node) {
-  ZoneHandleSet<Map> const& maps = CheckMapsParametersOf(node->op()).maps();
+  ZoneRefSet<Map> const& maps = CheckMapsParametersOf(node->op()).maps();
   Node* const object = NodeProperties::GetValueInput(node, 0);
   Node* const effect = NodeProperties::GetEffectInput(node);
   AbstractState const* state = node_states_.Get(effect);
   if (state == nullptr) return NoChange();
-  ZoneHandleSet<Map> object_maps;
+  ZoneRefSet<Map> object_maps;
   if (state->LookupMaps(object, &object_maps)) {
     if (maps.contains(object_maps)) return Replace(effect);
     // TODO(turbofan): Compute the intersection.
@@ -770,12 +791,12 @@ Reduction LoadElimination::ReduceCheckMaps(Node* node) {
 }
 
 Reduction LoadElimination::ReduceCompareMaps(Node* node) {
-  ZoneHandleSet<Map> const& maps = CompareMapsParametersOf(node->op());
+  ZoneRefSet<Map> const& maps = CompareMapsParametersOf(node->op());
   Node* const object = NodeProperties::GetValueInput(node, 0);
   Node* const effect = NodeProperties::GetEffectInput(node);
   AbstractState const* state = node_states_.Get(effect);
   if (state == nullptr) return NoChange();
-  ZoneHandleSet<Map> object_maps;
+  ZoneRefSet<Map> object_maps;
   if (state->LookupMaps(object, &object_maps)) {
     if (maps.contains(object_maps)) {
       Node* value = jsgraph()->TrueConstant();
@@ -794,8 +815,8 @@ Reduction LoadElimination::ReduceEnsureWritableFastElements(Node* node) {
   AbstractState const* state = node_states_.Get(effect);
   if (state == nullptr) return NoChange();
   // Check if the {elements} already have the fixed array map.
-  ZoneHandleSet<Map> elements_maps;
-  ZoneHandleSet<Map> fixed_array_maps(factory()->fixed_array_map());
+  ZoneRefSet<Map> elements_maps;
+  ZoneRefSet<Map> fixed_array_maps(broker()->fixed_array_map());
   if (state->LookupMaps(elements, &elements_maps) &&
       fixed_array_maps.contains(elements_maps)) {
     ReplaceWithValue(node, elements, effect);
@@ -823,12 +844,12 @@ Reduction LoadElimination::ReduceMaybeGrowFastElements(Node* node) {
   if (params.mode() == GrowFastElementsMode::kDoubleElements) {
     // We know that the resulting elements have the fixed double array map.
     state = state->SetMaps(
-        node, ZoneHandleSet<Map>(factory()->fixed_double_array_map()), zone());
+        node, ZoneRefSet<Map>(broker()->fixed_double_array_map()), zone());
   } else {
     // We know that the resulting elements have the fixed array map or the COW
     // version thereof (if we didn't grow and it was already COW before).
-    ZoneHandleSet<Map> fixed_array_maps(factory()->fixed_array_map());
-    fixed_array_maps.insert(factory()->fixed_cow_array_map(), zone());
+    ZoneRefSet<Map> fixed_array_maps(
+        {broker()->fixed_array_map(), broker()->fixed_cow_array_map()}, zone());
     state = state->SetMaps(node, fixed_array_maps, zone());
   }
   // Kill the previous elements on {object}.
@@ -845,8 +866,8 @@ Reduction LoadElimination::ReduceMaybeGrowFastElements(Node* node) {
 Reduction LoadElimination::ReduceTransitionElementsKind(Node* node) {
   ElementsTransition transition = ElementsTransitionOf(node->op());
   Node* const object = NodeProperties::GetValueInput(node, 0);
-  Handle<Map> source_map(transition.source());
-  Handle<Map> target_map(transition.target());
+  MapRef source_map(transition.source());
+  MapRef target_map(transition.target());
   Node* const effect = NodeProperties::GetEffectInput(node);
   AbstractState const* state = node_states_.Get(effect);
   if (state == nullptr) return NoChange();
@@ -861,14 +882,14 @@ Reduction LoadElimination::ReduceTransitionElementsKind(Node* node) {
           MaybeHandle<Name>(), zone());
       break;
   }
-  ZoneHandleSet<Map> object_maps;
+  ZoneRefSet<Map> object_maps;
   if (state->LookupMaps(object, &object_maps)) {
-    if (ZoneHandleSet<Map>(target_map).contains(object_maps)) {
+    if (ZoneRefSet<Map>(target_map).contains(object_maps)) {
       // The {object} already has the {target_map}, so this TransitionElements
       // {node} is fully redundant (independent of what {source_map} is).
       return Replace(effect);
     }
-    if (object_maps.contains(ZoneHandleSet<Map>(source_map))) {
+    if (object_maps.contains(ZoneRefSet<Map>(source_map))) {
       object_maps.remove(source_map, zone());
       object_maps.insert(target_map, zone());
       AliasStateInfo alias_info(state, object, source_map);
@@ -884,8 +905,8 @@ Reduction LoadElimination::ReduceTransitionElementsKind(Node* node) {
 
 Reduction LoadElimination::ReduceTransitionAndStoreElement(Node* node) {
   Node* const object = NodeProperties::GetValueInput(node, 0);
-  Handle<Map> double_map(DoubleMapParameterOf(node->op()));
-  Handle<Map> fast_map(FastMapParameterOf(node->op()));
+  MapRef double_map(DoubleMapParameterOf(node->op()));
+  MapRef fast_map(FastMapParameterOf(node->op()));
   Node* const effect = NodeProperties::GetEffectInput(node);
   AbstractState const* state = node_states_.Get(effect);
   if (state == nullptr) return NoChange();
@@ -893,7 +914,7 @@ Reduction LoadElimination::ReduceTransitionAndStoreElement(Node* node) {
   // We need to add the double and fast maps to the set of possible maps for
   // this object, because we don't know which of those we'll transition to.
   // Additionally, we should kill all alias information.
-  ZoneHandleSet<Map> object_maps;
+  ZoneRefSet<Map> object_maps;
   if (state->LookupMaps(object, &object_maps)) {
     object_maps.insert(double_map, zone());
     object_maps.insert(fast_map, zone());
@@ -917,9 +938,9 @@ Reduction LoadElimination::ReduceLoadField(Node* node,
   if (access.offset == HeapObject::kMapOffset &&
       access.base_is_tagged == kTaggedBase) {
     DCHECK(IsAnyTagged(access.machine_type.representation()));
-    ZoneHandleSet<Map> object_maps;
+    ZoneRefSet<Map> object_maps;
     if (state->LookupMaps(object, &object_maps) && object_maps.size() == 1) {
-      Node* value = jsgraph()->HeapConstant(object_maps[0]);
+      Node* value = jsgraph()->HeapConstantNoHole(object_maps[0].object());
       NodeProperties::SetType(value, Type::OtherInternal());
       ReplaceWithValue(node, value, effect);
       return Replace(value);
@@ -964,9 +985,8 @@ Reduction LoadElimination::ReduceLoadField(Node* node,
       state = state->AddField(object, field_index, info, zone());
     }
   }
-  Handle<Map> field_map;
-  if (access.map.ToHandle(&field_map)) {
-    state = state->SetMaps(node, ZoneHandleSet<Map>(field_map), zone());
+  if (access.map.has_value()) {
+    state = state->SetMaps(node, ZoneRefSet<Map>(*access.map), zone());
   }
   return UpdateState(node, state);
 }
@@ -986,8 +1006,8 @@ Reduction LoadElimination::ReduceStoreField(Node* node,
     Type const new_value_type = NodeProperties::GetType(new_value);
     if (new_value_type.IsHeapConstant()) {
       // Record the new {object} map information.
-      ZoneHandleSet<Map> object_maps(
-          new_value_type.AsHeapConstant()->Ref().AsMap().object());
+      ZoneRefSet<Map> object_maps(
+          new_value_type.AsHeapConstant()->Ref().AsMap());
       state = state->SetMaps(object, object_maps, zone());
     }
   } else {
@@ -1074,6 +1094,7 @@ Reduction LoadElimination::ReduceLoadElement(Node* node) {
     case MachineRepresentation::kFloat32:
     case MachineRepresentation::kCompressedPointer:
     case MachineRepresentation::kCompressed:
+    case MachineRepresentation::kIndirectPointer:
     case MachineRepresentation::kSandboxedPointer:
       // TODO(turbofan): Add support for doing the truncations.
       break;
@@ -1133,6 +1154,7 @@ Reduction LoadElimination::ReduceStoreElement(Node* node) {
     case MachineRepresentation::kCompressedPointer:
     case MachineRepresentation::kCompressed:
     case MachineRepresentation::kSandboxedPointer:
+    case MachineRepresentation::kIndirectPointer:
       // TODO(turbofan): Add support for doing the truncations.
       break;
     case MachineRepresentation::kFloat64:
@@ -1165,12 +1187,12 @@ LoadElimination::AbstractState const* LoadElimination::UpdateStateForPhi(
   // Check if all the inputs have the same maps.
   AbstractState const* input_state =
       node_states_.Get(NodeProperties::GetEffectInput(effect_phi, 0));
-  ZoneHandleSet<Map> object_maps;
+  ZoneRefSet<Map> object_maps;
   if (!input_state->LookupMaps(phi->InputAt(0), &object_maps)) return state;
   for (int i = 1; i < predecessor_count; i++) {
     input_state =
         node_states_.Get(NodeProperties::GetEffectInput(effect_phi, i));
-    ZoneHandleSet<Map> input_maps;
+    ZoneRefSet<Map> input_maps;
     if (!input_state->LookupMaps(phi->InputAt(i), &input_maps)) return state;
     if (input_maps != object_maps) return state;
   }
@@ -1319,10 +1341,9 @@ LoadElimination::AbstractState const* LoadElimination::ComputeLoopState(
           case IrOpcode::kTransitionElementsKind: {
             ElementsTransition transition = ElementsTransitionOf(current->op());
             Node* const object = NodeProperties::GetValueInput(current, 0);
-            ZoneHandleSet<Map> object_maps;
+            ZoneRefSet<Map> object_maps;
             if (!state->LookupMaps(object, &object_maps) ||
-                !ZoneHandleSet<Map>(transition.target())
-                     .contains(object_maps)) {
+                !ZoneRefSet<Map>(transition.target()).contains(object_maps)) {
               element_transitions_.push_back({transition, object});
             }
             break;
@@ -1440,11 +1461,17 @@ LoadElimination::IndexRange LoadElimination::FieldIndexOf(
     case MachineRepresentation::kMapWord:
     case MachineRepresentation::kCompressedPointer:
     case MachineRepresentation::kCompressed:
+    case MachineRepresentation::kIndirectPointer:
     case MachineRepresentation::kSandboxedPointer:
       break;
   }
   int representation_size = ElementSizeInBytes(rep);
   // We currently only track fields that are at least tagged pointer sized.
+  // We assume that indirect pointers are tagged pointer sized if we see them
+  // here since they should only ever be used in pointer compression
+  // configurations.
+  DCHECK(rep != MachineRepresentation::kIndirectPointer ||
+         representation_size == kTaggedSize);
   if (representation_size < kTaggedSize) return IndexRange::Invalid();
   DCHECK_EQ(0, representation_size % kTaggedSize);
 
