@@ -6,6 +6,11 @@
 
 #include "src/codegen/reloc-info.h"
 
+#ifdef DEBUG
+#include "src/objects/instruction-stream-inl.h"
+#include "src/objects/smi.h"
+#endif  // DEBUG
+
 namespace v8 {
 namespace internal {
 
@@ -22,9 +27,13 @@ const char* RootVisitor::RootName(Root root) {
   UNREACHABLE();
 }
 
-void ObjectVisitor::VisitRelocInfo(RelocIterator* it) {
+void ObjectVisitor::VisitRelocInfo(Tagged<InstructionStream> host,
+                                   RelocIterator* it) {
+  // RelocInfo iteration is only valid for fully-initialized InstructionStream
+  // objects. Callers must ensure this.
+  DCHECK(host->IsFullyInitialized());
   for (; !it->done(); it->next()) {
-    it->rinfo()->Visit(this);
+    it->rinfo()->Visit(host, this);
   }
 }
 
