@@ -128,7 +128,7 @@ function Agent(options) {
     }
 
     const requests = this.requests[name];
-    if (requests && requests.length) {
+    if (requests?.length) {
       const req = requests.shift();
       const reqAsyncRes = req[kRequestAsyncResource];
       if (reqAsyncRes) {
@@ -246,9 +246,7 @@ Agent.prototype.addRequest = function addRequest(req, options, port/* legacy */,
   normalizeServerName(options, req);
 
   const name = this.getName(options);
-  if (!this.sockets[name]) {
-    this.sockets[name] = [];
-  }
+  this.sockets[name] ||= [];
 
   const freeSockets = this.freeSockets[name];
   let socket;
@@ -284,9 +282,7 @@ Agent.prototype.addRequest = function addRequest(req, options, port/* legacy */,
   } else {
     debug('wait for socket');
     // We are over limit so we'll add it to the queue.
-    if (!this.requests[name]) {
-      this.requests[name] = [];
-    }
+    this.requests[name] ||= [];
 
     // Used to create sockets for pending requests from different origin
     req[kRequestOptions] = options;
@@ -313,9 +309,7 @@ Agent.prototype.createSocket = function createSocket(req, options, cb) {
   const oncreate = once((err, s) => {
     if (err)
       return cb(err);
-    if (!this.sockets[name]) {
-      this.sockets[name] = [];
-    }
+    this.sockets[name] ||= [];
     this.sockets[name].push(s);
     this.totalSocketCount++;
     debug('sockets', name, this.sockets[name].length, this.totalSocketCount);
@@ -347,7 +341,7 @@ function calculateServerName(options, req) {
     // abc:123 => abc
     // [::1] => ::1
     // [::1]:123 => ::1
-    if (hostHeader.startsWith('[')) {
+    if (hostHeader[0] === '[') {
       const index = hostHeader.indexOf(']');
       if (index === -1) {
         // Leading '[', but no ']'. Need to do something...
@@ -437,7 +431,7 @@ Agent.prototype.removeSocket = function removeSocket(s, options) {
   }
 
   let req;
-  if (this.requests[name] && this.requests[name].length) {
+  if (this.requests[name]?.length) {
     debug('removeSocket, have a request, make a socket');
     req = this.requests[name][0];
   } else {
@@ -449,7 +443,7 @@ Agent.prototype.removeSocket = function removeSocket(s, options) {
     for (let i = 0; i < keys.length; i++) {
       const prop = keys[i];
       // Check whether this specific origin is already at maxSockets
-      if (this.sockets[prop] && this.sockets[prop].length) break;
+      if (this.sockets[prop]?.length) break;
       debug('removeSocket, have a request with different origin,' +
         ' make a socket');
       req = this.requests[prop][0];
