@@ -34,8 +34,8 @@ const {
   FunctionPrototypeBind,
   FunctionPrototypeCall,
   NumberMAX_SAFE_INTEGER,
-  ObjectDefineProperty,
   ObjectDefineProperties,
+  ObjectDefineProperty,
   ObjectGetPrototypeOf,
   ObjectSetPrototypeOf,
   Promise,
@@ -46,32 +46,31 @@ const {
   String,
   StringPrototypeSplit,
   Symbol,
-  SymbolFor,
   SymbolAsyncIterator,
+  SymbolFor,
 } = primordials;
 const kRejection = SymbolFor('nodejs.rejection');
 
-const { SymbolDispose, kEmptyObject } = require('internal/util');
+const { SymbolDispose, kEmptyObject, spliceOne } = require('internal/util');
 
 const {
   inspect,
   identicalSequenceRange,
 } = require('internal/util/inspect');
 
-let spliceOne;
 let FixedQueue;
 let kFirstEventParam;
 let kResistStopPropagation;
 
 const {
   AbortError,
-  kEnhanceStackBeforeInspector,
   codes: {
     ERR_INVALID_ARG_TYPE,
     ERR_INVALID_THIS,
     ERR_UNHANDLED_ERROR,
   },
   genericNodeError,
+  kEnhanceStackBeforeInspector,
 } = require('internal/errors');
 
 const {
@@ -80,6 +79,7 @@ const {
   validateBoolean,
   validateFunction,
   validateNumber,
+  validateObject,
   validateString,
 } = require('internal/validators');
 const { addAbortListener } = require('internal/events/abort_listener');
@@ -287,12 +287,6 @@ ObjectDefineProperty(EventEmitter, 'defaultMaxListeners', {
   },
 });
 
-function hasEventListener(self, type) {
-  if (type === undefined)
-    return self._events !== undefined;
-  return self._events !== undefined && self._events[type] !== undefined;
-}
-
 ObjectDefineProperties(EventEmitter, {
   kMaxEventTargetListeners: {
     __proto__: null,
@@ -355,7 +349,7 @@ EventEmitter.init = function(opts) {
     this[kShapeMode] = true;
   }
 
-  this._maxListeners = this._maxListeners || undefined;
+  this._maxListeners ||= undefined;
 
 
   if (opts?.captureRejections) {
@@ -475,7 +469,7 @@ EventEmitter.prototype.emit = function emit(type, ...args) {
   if (events !== undefined) {
     if (doError && events[kErrorMonitor] !== undefined)
       this.emit(kErrorMonitor, ...args);
-    doError = (doError && events.error === undefined);
+    doError &&= events.error === undefined;
   } else if (!doError)
     return false;
 
@@ -686,11 +680,13 @@ EventEmitter.prototype.removeListener =
     function removeListener(type, listener) {
       checkListener(listener);
 
-      if (!hasEventListener(this, type))
+      const events = this._events;
+      if (events === undefined)
         return this;
 
-      const events = this._events;
       const list = events[type];
+      if (list === undefined)
+        return this;
 
       if (list === listener || list.listener === listener) {
         this._eventsCount -= 1;
@@ -720,8 +716,6 @@ EventEmitter.prototype.removeListener =
         if (position === 0)
           list.shift();
         else {
-          if (spliceOne === undefined)
-            spliceOne = require('internal/util').spliceOne;
           spliceOne(list, position);
         }
 
@@ -746,9 +740,9 @@ EventEmitter.prototype.off = EventEmitter.prototype.removeListener;
  */
 EventEmitter.prototype.removeAllListeners =
     function removeAllListeners(type) {
-      if (!hasEventListener(this))
-        return this;
       const events = this._events;
+      if (events === undefined)
+        return this;
 
       // Not listening for removeListener, no need to emit
       if (events.removeListener === undefined) {
@@ -793,10 +787,14 @@ EventEmitter.prototype.removeAllListeners =
     };
 
 function _listeners(target, type, unwrap) {
-  if (!hasEventListener(target, type))
+  const events = target._events;
+
+  if (events === undefined)
     return [];
 
-  const evlistener = target._events[type];
+  const evlistener = events[type];
+  if (evlistener === undefined)
+    return [];
 
   if (typeof evlistener === 'function')
     return unwrap ? [evlistener.listener || evlistener] : [evlistener];
@@ -970,10 +968,11 @@ function getMaxListeners(emitterOrTarget) {
  * @returns {Promise}
  */
 async function once(emitter, name, options = kEmptyObject) {
-  const signal = options?.signal;
+  validateObject(options, 'options');
+  const { signal } = options;
   validateAbortSignal(signal, 'options.signal');
   if (signal?.aborted)
-    throw new AbortError(undefined, { cause: signal?.reason });
+    throw new AbortError(undefined, { cause: signal.reason });
   return new Promise((resolve, reject) => {
     const errorListener = (err) => {
       emitter.removeListener(name, resolver);
@@ -1057,10 +1056,11 @@ function eventTargetAgnosticAddListener(emitter, name, listener, flags) {
  */
 function on(emitter, event, options = kEmptyObject) {
   // Parameters validation
+  validateObject(options, 'options');
   const signal = options.signal;
   validateAbortSignal(signal, 'options.signal');
   if (signal?.aborted)
-    throw new AbortError(undefined, { cause: signal?.reason });
+    throw new AbortError(undefined, { cause: signal.reason });
   // Support both highWaterMark and highWatermark for backward compatibility
   const highWatermark = options.highWaterMark ?? options.highWatermark ?? NumberMAX_SAFE_INTEGER;
   validateInteger(highWatermark, 'options.highWaterMark', 1);
