@@ -3,9 +3,10 @@
 const {
   ArrayPrototypeMap,
   Boolean,
+  FunctionPrototypeCall,
   JSONParse,
-  ObjectPrototypeHasOwnProperty,
   ObjectKeys,
+  ObjectPrototypeHasOwnProperty,
   ReflectApply,
   SafeArrayIterator,
   SafeMap,
@@ -17,16 +18,6 @@ const {
   globalThis: { WebAssembly },
 } = primordials;
 
-/** @type {import('internal/util/types')} */
-let _TYPES = null;
-/**
- * Lazily loads and returns the internal/util/types module.
- */
-function lazyTypes() {
-  if (_TYPES !== null) { return _TYPES; }
-  return _TYPES = require('internal/util/types');
-}
-
 const {
   compileFunctionForCJSLoader,
 } = internalBinding('contextify');
@@ -36,13 +27,17 @@ const assert = require('internal/assert');
 const { readFileSync } = require('fs');
 const { dirname, extname, isAbsolute } = require('path');
 const {
+  assertBufferSource,
   loadBuiltinModule,
+  stringify,
   stripBOM,
   urlToFilename,
 } = require('internal/modules/helpers');
+const { stripTypeScriptModuleTypes } = require('internal/modules/typescript');
 const {
   kIsCachedByESMLoader,
   Module: CJSModule,
+  wrapModuleLoad,
   kModuleSource,
   kModuleExport,
   kModuleExportNames,
@@ -51,10 +46,9 @@ const { fileURLToPath, pathToFileURL, URL } = require('internal/url');
 let debug = require('internal/util/debuglog').debuglog('esm', (fn) => {
   debug = fn;
 });
-const { emitExperimentalWarning, kEmptyObject, setOwnProperty } = require('internal/util');
+const { emitExperimentalWarning, kEmptyObject, setOwnProperty, isWindows } = require('internal/util');
 const {
   ERR_UNKNOWN_BUILTIN_MODULE,
-  ERR_INVALID_RETURN_PROPERTY_VALUE,
 } = require('internal/errors').codes;
 const { maybeCacheSourceMap } = require('internal/source_map/source_map_cache');
 const moduleWrap = internalBinding('module_wrap');
@@ -87,44 +81,6 @@ function initCJSParseSync() {
 const translators = new SafeMap();
 exports.translators = translators;
 
-let DECODER = null;
-/**
- * Asserts that the given body is a buffer source (either a string, array buffer, or typed array).
- * Throws an error if the body is not a buffer source.
- * @param {string | ArrayBufferView | ArrayBuffer} body - The body to check.
- * @param {boolean} allowString - Whether or not to allow a string as a valid buffer source.
- * @param {string} hookName - The name of the hook being called.
- * @throws {ERR_INVALID_RETURN_PROPERTY_VALUE} If the body is not a buffer source.
- */
-function assertBufferSource(body, allowString, hookName) {
-  if (allowString && typeof body === 'string') {
-    return;
-  }
-  const { isArrayBufferView, isAnyArrayBuffer } = lazyTypes();
-  if (isArrayBufferView(body) || isAnyArrayBuffer(body)) {
-    return;
-  }
-  throw new ERR_INVALID_RETURN_PROPERTY_VALUE(
-    `${allowString ? 'string, ' : ''}array buffer, or typed array`,
-    hookName,
-    'source',
-    body,
-  );
-}
-
-/**
- * Converts a buffer or buffer-like object to a string.
- * @param {string | ArrayBuffer | ArrayBufferView} body - The buffer or buffer-like object to convert to a string.
- * @returns {string} The resulting string.
- */
-function stringify(body) {
-  if (typeof body === 'string') { return body; }
-  assertBufferSource(body, false, 'load');
-  const { TextDecoder } = require('internal/encoding');
-  DECODER = DECODER === null ? new TextDecoder() : DECODER;
-  return DECODER.decode(body);
-}
-
 /**
  * Converts a URL to a file path if the URL protocol is 'file:'.
  * @param {string} url - The URL to convert.
@@ -180,7 +136,7 @@ function loadCJSModule(module, source, url, filename, isMain) {
           importAttributes = { __proto__: null, type: 'json' };
           break;
         case '.node':
-          return CJSModule._load(specifier, module);
+          return wrapModuleLoad(specifier, module);
         default:
             // fall through
       }
@@ -270,7 +226,7 @@ translators.set('commonjs-sync', function requireCommonJS(url, source, isMain) {
 
   return createCJSModuleWrap(url, source, isMain, (module, source, url, filename, isMain) => {
     assert(module === CJSModule._cache[filename]);
-    CJSModule._load(filename, null, isMain);
+    wrapModuleLoad(filename, null, isMain);
   });
 });
 
@@ -283,6 +239,14 @@ translators.set('require-commonjs', (url, source, isMain) => {
   return createCJSModuleWrap(url, source);
 });
 
+// Handle CommonJS modules referenced by `require` calls.
+// This translator function must be sync, as `require` is sync.
+translators.set('require-commonjs-typescript', (url, source, isMain) => {
+  assert(cjsParse);
+  const code = stripTypeScriptModuleTypes(stringify(source), url);
+  return createCJSModuleWrap(url, code);
+});
+
 // Handle CommonJS modules referenced by `import` statements or expressions,
 // or as the initial entry point when the ESM loader handles a CommonJS entry.
 translators.set('commonjs', function commonjsStrategy(url, source, isMain) {
@@ -295,7 +259,7 @@ translators.set('commonjs', function commonjsStrategy(url, source, isMain) {
   // obtained by calling the monkey-patchable CJS loader.
   const cjsLoader = source == null ? (module, source, url, filename, isMain) => {
     assert(module === CJSModule._cache[filename]);
-    CJSModule._load(filename, undefined, isMain);
+    wrapModuleLoad(filename, undefined, isMain);
   } : loadCJSModule;
 
   try {
@@ -395,7 +359,6 @@ translators.set('builtin', function builtinStrategy(url) {
 });
 
 // Strategy for loading a JSON file
-const isWindows = process.platform === 'win32';
 translators.set('json', function jsonStrategy(url, source) {
   assertBufferSource(source, true, 'load');
   debug(`Loading JSONModule ${url}`);
@@ -410,7 +373,7 @@ translators.set('json', function jsonStrategy(url, source) {
     modulePath = isWindows ?
       StringPrototypeReplaceAll(pathname, '/', '\\') : pathname;
     module = CJSModule._cache[modulePath];
-    if (module && module.loaded) {
+    if (module?.loaded) {
       const exports = module.exports;
       return new ModuleWrap(url, undefined, ['default'], function() {
         this.setExport('default', exports);
@@ -424,7 +387,7 @@ translators.set('json', function jsonStrategy(url, source) {
     // export, we have to check again if the module already exists or not.
     // TODO: remove CJS loader from here as well.
     module = CJSModule._cache[modulePath];
-    if (module && module.loaded) {
+    if (module?.loaded) {
       const exports = module.exports;
       return new ModuleWrap(url, undefined, ['default'], function() {
         this.setExport('default', exports);
@@ -490,3 +453,19 @@ translators.set('wasm', async function(url, source) {
     }
   }).module;
 });
+
+// Strategy for loading a commonjs TypeScript module
+translators.set('commonjs-typescript', function(url, source) {
+  assertBufferSource(source, true, 'load');
+  const code = stripTypeScriptModuleTypes(stringify(source), url);
+  debug(`Translating TypeScript ${url}`);
+  return FunctionPrototypeCall(translators.get('commonjs'), this, url, code, false);
+});
+
+// Strategy for loading an esm TypeScript module
+translators.set('module-typescript', function(url, source) {
+  assertBufferSource(source, true, 'load');
+  const code = stripTypeScriptModuleTypes(stringify(source), url);
+  debug(`Translating TypeScript ${url}`);
+  return FunctionPrototypeCall(translators.get('module'), this, url, code, false);
+});
