@@ -13,9 +13,10 @@ const {
   ArrayPrototypeSplice,
   BigUint64Array,
   Float64Array,
+  FunctionPrototypeCall,
   NumberMAX_SAFE_INTEGER,
-  ObjectFreeze,
   ObjectDefineProperty,
+  ObjectFreeze,
   ReflectApply,
   RegExpPrototypeExec,
   SafeArrayIterator,
@@ -26,6 +27,7 @@ const {
   StringPrototypeReplace,
   StringPrototypeSlice,
   Symbol,
+  SymbolFor,
   SymbolIterator,
 } = primordials;
 
@@ -45,10 +47,11 @@ const {
   validateNumber,
   validateObject,
 } = require('internal/validators');
-const { getValidatedPath } = require('internal/fs/utils');
-const { toNamespacedPath } = require('path');
+
 const constants = internalBinding('constants').os.signals;
 
+let getValidatedPath; // We need to lazy load it because of the circular dependency.
+
 const kInternal = Symbol('internal properties');
 
 function assert(x, msg) {
@@ -172,9 +175,6 @@ function wrapProcessMethods(binding) {
   memoryUsage.rss = rss;
 
   function exit(code) {
-    const { handleProcessExit } = require('internal/modules/run_main');
-    process.off('exit', handleProcessExit);
-
     if (arguments.length !== 0) {
       process.exitCode = code;
     }
@@ -213,7 +213,7 @@ function wrapProcessMethods(binding) {
       // it's monkey-patched by tests.
       err = process._kill(pid, sig);
     } else {
-      sig = sig || 'SIGTERM';
+      sig ||= 'SIGTERM';
       if (constants[sig]) {
         err = process._kill(pid, constants[sig]);
       } else {
@@ -256,14 +256,14 @@ function wrapProcessMethods(binding) {
    */
   function loadEnvFile(path = undefined) { // Provide optional value so that `loadEnvFile.length` returns 0
     if (path != null) {
+      getValidatedPath ??= require('internal/fs/utils').getValidatedPath;
       path = getValidatedPath(path);
-      _loadEnvFile(toNamespacedPath(path));
+      _loadEnvFile(path);
     } else {
       _loadEnvFile();
     }
   }
 
-
   return {
     _rawDebug,
     cpuUsage,
@@ -415,16 +415,29 @@ let traceEventsAsyncHook;
 // Dynamically enable/disable the traceEventsAsyncHook
 function toggleTraceCategoryState(asyncHooksEnabled) {
   if (asyncHooksEnabled) {
-    if (!traceEventsAsyncHook) {
-      traceEventsAsyncHook =
-        require('internal/trace_events_async_hooks').createHook();
-    }
+    traceEventsAsyncHook ||= require('internal/trace_events_async_hooks').createHook();
     traceEventsAsyncHook.enable();
   } else if (traceEventsAsyncHook) {
     traceEventsAsyncHook.disable();
   }
 }
 
+const { arch, platform, version } = process;
+
+let refSymbol;
+function ref(maybeRefable) {
+  if (maybeRefable == null) return;
+  const fn = maybeRefable[refSymbol ??= SymbolFor('nodejs.ref')] || maybeRefable.ref;
+  if (typeof fn === 'function') FunctionPrototypeCall(fn, maybeRefable);
+}
+
+let unrefSymbol;
+function unref(maybeRefable) {
+  if (maybeRefable == null) return;
+  const fn = maybeRefable[unrefSymbol ??= SymbolFor('nodejs.unref')] || maybeRefable.unref;
+  if (typeof fn === 'function') FunctionPrototypeCall(fn, maybeRefable);
+}
+
 module.exports = {
   toggleTraceCategoryState,
   assert,
@@ -432,4 +445,9 @@ module.exports = {
   wrapProcessMethods,
   hrtime,
   hrtimeBigInt,
+  arch,
+  platform,
+  version,
+  ref,
+  unref,
 };
