@@ -25,6 +25,7 @@ const {
   ArrayIsArray,
   ArrayPrototypeJoin,
   ArrayPrototypePop,
+  ArrayPrototypePush,
   Date,
   DatePrototypeGetDate,
   DatePrototypeGetHours,
@@ -48,14 +49,14 @@ const {
 } = primordials;
 
 const {
+  ErrnoException,
+  ExceptionWithHostPort,
   codes: {
     ERR_FALSY_VALUE_REJECTION,
     ERR_INVALID_ARG_TYPE,
     ERR_OUT_OF_RANGE,
   },
   isErrorStackTraceLimitWritable,
-  ErrnoException,
-  ExceptionWithHostPort,
 } = require('internal/errors');
 const {
   format,
@@ -70,6 +71,7 @@ const {
   validateNumber,
   validateString,
   validateOneOf,
+  validateObject,
 } = require('internal/validators');
 const { isBuffer } = require('buffer').Buffer;
 const {
@@ -84,13 +86,16 @@ function lazyUtilColors() {
   utilColors ??= require('internal/util/colors');
   return utilColors;
 }
+const { getOptionValue } = require('internal/options');
 
 const binding = internalBinding('util');
 
 const {
   deprecate,
+  getLazy,
   getSystemErrorMap,
   getSystemErrorName: internalErrorName,
+  getSystemErrorMessage: internalErrorMessage,
   promisify,
   defineLazyProperties,
 } = require('internal/util');
@@ -298,9 +303,7 @@ let console;
  * @type {(...args: any[]) => void}
  */
 function log(...args) {
-  if (!console) {
-    console = require('internal/console/global');
-  }
+  console ??= require('internal/console/global');
   console.log('%s - %s', timestamp(), format(...args));
 }
 
@@ -415,6 +418,18 @@ function callbackify(original) {
   return callbackified;
 }
 
+/**
+ * @param {number} err
+ * @returns {string}
+ */
+function getSystemErrorMessage(err) {
+  validateNumber(err, 'err');
+  if (err >= 0 || !NumberIsSafeInteger(err)) {
+    throw new ERR_OUT_OF_RANGE('err', 'a negative integer', err);
+  }
+  return internalErrorMessage(err);
+}
+
 /**
  * @param {number} err
  * @returns {string}
@@ -461,11 +476,101 @@ function parseEnv(content) {
   return binding.parseEnv(content);
 }
 
+const lazySourceMap = getLazy(() => require('internal/source_map/source_map_cache'));
+
+/**
+ * @typedef {object} CallSite // The call site
+ * @property {string} scriptName // The name of the resource that contains the
+ * script for the function for this StackFrame
+ * @property {string} functionName // The name of the function associated with this stack frame
+ * @property {number} lineNumber // The number, 1-based, of the line for the associate function call
+ * @property {number} columnNumber // The 1-based column offset on the line for the associated function call
+ */
+
+/**
+ * @param {CallSite} callSite // The call site object to reconstruct from source map
+ * @returns {CallSite | undefined} // The reconstructed call site object
+ */
+function reconstructCallSite(callSite) {
+  const { scriptName, lineNumber, columnNumber } = callSite;
+  const sourceMap = lazySourceMap().findSourceMap(scriptName);
+  if (!sourceMap) return;
+  const entry = sourceMap.findEntry(lineNumber - 1, columnNumber - 1);
+  if (!entry?.originalSource) return;
+  return {
+    __proto__: null,
+    // If the name is not found, it is an empty string to match the behavior of `util.getCallSite()`
+    functionName: entry.name ?? '',
+    scriptName: entry.originalSource,
+    lineNumber: entry.originalLine + 1,
+    column: entry.originalColumn + 1,
+    columnNumber: entry.originalColumn + 1,
+  };
+}
+
+/**
+ *
+ * The call site array to map
+ * @param {CallSite[]} callSites
+ * Array of objects with the reconstructed call site
+ * @returns {CallSite[]}
+ */
+function mapCallSite(callSites) {
+  const result = [];
+  for (let i = 0; i < callSites.length; ++i) {
+    const callSite = callSites[i];
+    const found = reconstructCallSite(callSite);
+    ArrayPrototypePush(result, found ?? callSite);
+  }
+  return result;
+}
+
+/**
+ * @typedef {object} CallSiteOptions // The call site options
+ * @property {boolean} sourceMap // Enable source map support
+ */
+
+/**
+ * Returns the callSite
+ * @param {number} frameCount
+ * @param {CallSiteOptions} options
+ * @returns {CallSite[]}
+ */
+function getCallSites(frameCount = 10, options) {
+  // If options is not provided check if frameCount is an object
+  if (options === undefined) {
+    if (typeof frameCount === 'object') {
+      // If frameCount is an object, it is the options object
+      options = frameCount;
+      validateObject(options, 'options');
+      validateBoolean(options.sourceMap, 'options.sourceMap');
+      frameCount = 10;
+    } else {
+      // If options is not provided, set it to an empty object
+      options = {};
+    };
+  } else {
+    // If options is provided, validate it
+    validateObject(options, 'options');
+    validateBoolean(options.sourceMap, 'options.sourceMap');
+  }
+
+  // Using kDefaultMaxCallStackSizeToCapture as reference
+  validateNumber(frameCount, 'frameCount', 1, 200);
+  // If options.sourceMaps is true or if sourceMaps are enabled but the option.sourceMaps is not set explictly to false
+  if (options.sourceMap === true || (getOptionValue('--enable-source-maps') && options.sourceMap !== false)) {
+    return mapCallSite(binding.getCallSites(frameCount));
+  }
+  return binding.getCallSites(frameCount);
+};
+
 // Keep the `exports =` so that various functions can still be monkeypatched
 module.exports = {
   _errnoException,
   _exceptionWithHostPort,
-  _extend,
+  _extend: deprecate(_extend,
+                     'The `util._extend` API is deprecated. Please use Object.assign() instead.',
+                     'DEP0060'),
   callbackify,
   debug: debuglog,
   debuglog,
@@ -473,13 +578,26 @@ module.exports = {
   format,
   styleText,
   formatWithOptions,
+  // Deprecated getCallSite.
+  // This API can be removed in next semver-minor release.
+  getCallSite: deprecate(getCallSites,
+                         'The `util.getCallSite` API has been renamed to `util.getCallSites()`.',
+                         'ExperimentalWarning'),
+  getCallSites,
   getSystemErrorMap,
   getSystemErrorName,
+  getSystemErrorMessage,
   inherits,
   inspect,
-  isArray: ArrayIsArray,
-  isBoolean,
-  isBuffer,
+  isArray: deprecate(ArrayIsArray,
+                     'The `util.isArray` API is deprecated. Please use `Array.isArray()` instead.',
+                     'DEP0044'),
+  isBoolean: deprecate(isBoolean,
+                       'The `util.isBoolean` API is deprecated.  Please use `typeof arg === "boolean"` instead.',
+                       'DEP0045'),
+  isBuffer: deprecate(isBuffer,
+                      'The `util.isBuffer` API is deprecated. Please use `Buffer.isBuffer()` instead.',
+                      'DEP0046'),
   isDeepStrictEqual(a, b) {
     if (internalDeepEqual === undefined) {
       internalDeepEqual = require('internal/util/comparisons')
@@ -487,19 +605,52 @@ module.exports = {
     }
     return internalDeepEqual(a, b);
   },
-  isNull,
-  isNullOrUndefined,
-  isNumber,
-  isString,
-  isSymbol,
-  isUndefined,
-  isRegExp: types.isRegExp,
-  isObject,
-  isDate: types.isDate,
-  isError,
-  isFunction,
-  isPrimitive,
-  log,
+  isNull: deprecate(isNull,
+                    'The `util.isNull` API is deprecated. Please use `arg === null` instead.',
+                    'DEP0050'),
+  isNullOrUndefined: deprecate(isNullOrUndefined,
+                               'The `util.isNullOrUndefined` API is deprecated. ' +
+                               'Please use `arg === null || arg === undefined` instead.',
+                               'DEP0051'),
+  isNumber: deprecate(isNumber,
+                      'The `util.isNumber` API is deprecated. Please use `typeof arg === "number"` instead.',
+                      'DEP0052'),
+  isString: deprecate(isString,
+                      'The `util.isString` API is deprecated.  Please use `typeof arg === "string"` instead.',
+                      'DEP0056'),
+  isSymbol: deprecate(isSymbol,
+                      'The `util.isSymbol` API is deprecated.  Please use `arg === "symbol"` instead.',
+                      'DEP0057'),
+  isUndefined: deprecate(isUndefined,
+                         'The `util.isUndefined` API is deprecated. Please use `arg === undefined` instead.',
+                         'DEP0058'),
+  isRegExp: deprecate(types.isRegExp,
+                      'The `util.isRegExp` API is deprecated. Please use `arg instanceof RegExp` instead.',
+                      'DEP0055'),
+  isObject: deprecate(isObject,
+                      'The `util.isObject` API is deprecated. ' +
+                      'Please use `arg !== null && typeof arg === "object"` instead.',
+                      'DEP0053'),
+  isDate: deprecate(types.isDate,
+                    'The `util.isDate` API is deprecated.  Please use `arg instanceof Date` instead.',
+                    'DEP0047'),
+  isError: deprecate(isError,
+                     'The `util.isError` API is deprecated. ' +
+                     'Please use `ObjectPrototypeToString(e) === "[object Error]" ' +
+                     '|| e instanceof Error` instead.',
+                     'DEP0048'),
+  isFunction: deprecate(isFunction,
+                        'The `util.isFunction` API is deprecated.  Please use `typeof arg === "function"` instead.',
+                        'DEP0049'),
+  isPrimitive: deprecate(isPrimitive,
+                         'The `util.isPrimitive` API is deprecated. ' +
+                         'Please use `arg === null || ' +
+                         '(typeof arg !== "object" && typeof arg !== "function")` instead.',
+                         'DEP0054'),
+  log: deprecate(log,
+                 'The `util.log API is deprecated. ' +
+                 'Please use console.log() with a custom formatter or a third-party logger instead.',
+                 'DEP0059'),
   promisify,
   stripVTControlCharacters,
   toUSVString(input) {
