@@ -22,10 +22,10 @@
 #include "cares_wrap.h"
 #include "ada.h"
 #include "async_wrap-inl.h"
-#include "base64-inl.h"
 #include "base_object-inl.h"
 #include "env-inl.h"
 #include "memory_tracker-inl.h"
+#include "nbytes.h"
 #include "node.h"
 #include "node_errors.h"
 #include "node_external_reference.h"
@@ -59,6 +59,7 @@ namespace cares_wrap {
 using v8::Array;
 using v8::Context;
 using v8::EscapableHandleScope;
+using v8::Exception;
 using v8::FunctionCallbackInfo;
 using v8::FunctionTemplate;
 using v8::HandleScope;
@@ -66,7 +67,9 @@ using v8::Int32;
 using v8::Integer;
 using v8::Isolate;
 using v8::Just;
+using v8::JustVoid;
 using v8::Local;
+using v8::LocalVector;
 using v8::Maybe;
 using v8::Nothing;
 using v8::Null;
@@ -158,7 +161,7 @@ void ares_sockstate_cb(void* data, ares_socket_t sock, int read, int write) {
 Local<Array> HostentToNames(Environment* env, struct hostent* host) {
   EscapableHandleScope scope(env->isolate());
 
-  std::vector<Local<Value>> names;
+  LocalVector<Value> names(env->isolate());
 
   for (uint32_t i = 0; host->h_aliases[i] != nullptr; ++i)
     names.emplace_back(OneByteString(env->isolate(), host->h_aliases[i]));
@@ -587,11 +590,11 @@ int ParseSoaReply(
         return ARES_EBADRESP;
       }
 
-      const unsigned int serial = ReadUint32BE(ptr + 0 * 4);
-      const unsigned int refresh = ReadUint32BE(ptr + 1 * 4);
-      const unsigned int retry = ReadUint32BE(ptr + 2 * 4);
-      const unsigned int expire = ReadUint32BE(ptr + 3 * 4);
-      const unsigned int minttl = ReadUint32BE(ptr + 4 * 4);
+      const unsigned int serial = nbytes::ReadUint32BE(ptr + 0 * 4);
+      const unsigned int refresh = nbytes::ReadUint32BE(ptr + 1 * 4);
+      const unsigned int retry = nbytes::ReadUint32BE(ptr + 2 * 4);
+      const unsigned int expire = nbytes::ReadUint32BE(ptr + 3 * 4);
+      const unsigned int minttl = nbytes::ReadUint32BE(ptr + 4 * 4);
 
       Local<Object> soa_record = Object::New(env->isolate());
       soa_record->Set(env->context(),
@@ -711,7 +714,6 @@ void ChannelWrap::Setup() {
   options.sock_state_cb_data = this;
   options.timeout = timeout_;
   options.tries = tries_;
-  options.qcache_max_ttl = 0;
 
   int r;
   if (!library_inited_) {
@@ -887,8 +889,9 @@ int SoaTraits::Send(QueryWrap<SoaTraits>* wrap, const char* name) {
 int AnyTraits::Parse(
     QueryAnyWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
+  }
 
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
@@ -1054,8 +1057,9 @@ int AnyTraits::Parse(
 int ATraits::Parse(
     QueryAWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
+  }
 
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
@@ -1088,8 +1092,9 @@ int ATraits::Parse(
 int AaaaTraits::Parse(
     QueryAaaaWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
+  }
 
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
@@ -1122,8 +1127,9 @@ int AaaaTraits::Parse(
 int CaaTraits::Parse(
     QueryCaaWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
+  }
 
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
@@ -1144,8 +1150,9 @@ int CaaTraits::Parse(
 int CnameTraits::Parse(
     QueryCnameWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
+  }
 
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
@@ -1167,8 +1174,9 @@ int CnameTraits::Parse(
 int MxTraits::Parse(
     QueryMxWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
+  }
 
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
@@ -1190,8 +1198,9 @@ int MxTraits::Parse(
 int NsTraits::Parse(
     QueryNsWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
+  }
 
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
@@ -1213,8 +1222,9 @@ int NsTraits::Parse(
 int TxtTraits::Parse(
     QueryTxtWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
+  }
 
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
@@ -1235,8 +1245,9 @@ int TxtTraits::Parse(
 int SrvTraits::Parse(
     QuerySrvWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
+  }
 
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
@@ -1257,9 +1268,9 @@ int SrvTraits::Parse(
 int PtrTraits::Parse(
     QueryPtrWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
-
+  }
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
 
@@ -1281,9 +1292,9 @@ int PtrTraits::Parse(
 int NaptrTraits::Parse(
     QueryNaptrWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
-
+  }
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
 
@@ -1303,9 +1314,9 @@ int NaptrTraits::Parse(
 int SoaTraits::Parse(
     QuerySoaWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(response->is_host))
+  if (response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
-
+  }
   unsigned char* buf = response->buf.data;
   int len = response->buf.size;
 
@@ -1383,9 +1394,9 @@ int ReverseTraits::Send(GetHostByAddrWrap* wrap, const char* name) {
 int ReverseTraits::Parse(
     GetHostByAddrWrap* wrap,
     const std::unique_ptr<ResponseData>& response) {
-  if (UNLIKELY(!response->is_host))
+  if (!response->is_host) [[unlikely]] {
     return ARES_EBADRESP;
-
+  }
   struct hostent* host = response->host.get();
 
   Environment* env = wrap->env();
@@ -1446,7 +1457,7 @@ void AfterGetAddrInfo(uv_getaddrinfo_t* req, int status, struct addrinfo* res) {
   if (status == 0) {
     Local<Array> results = Array::New(env->isolate());
 
-    auto add = [&] (bool want_ipv4, bool want_ipv6) -> Maybe<bool> {
+    auto add = [&](bool want_ipv4, bool want_ipv6) -> Maybe<void> {
       for (auto p = res; p != nullptr; p = p->ai_next) {
         CHECK_EQ(p->ai_socktype, SOCK_STREAM);
 
@@ -1467,21 +1478,21 @@ void AfterGetAddrInfo(uv_getaddrinfo_t* req, int status, struct addrinfo* res) {
 
         Local<String> s = OneByteString(env->isolate(), ip);
         if (results->Set(env->context(), n, s).IsNothing())
-          return Nothing<bool>();
+          return Nothing<void>();
         n++;
       }
-      return Just(true);
+      return JustVoid();
     };
 
     switch (order) {
       case DNS_ORDER_IPV4_FIRST:
-        if (add(true, false).IsNothing()) return;
-        if (add(false, true).IsNothing()) return;
+        if (add(true, false).IsNothing() || add(false, true).IsNothing())
+          return;
 
         break;
       case DNS_ORDER_IPV6_FIRST:
-        if (add(false, true).IsNothing()) return;
-        if (add(true, false).IsNothing()) return;
+        if (add(false, true).IsNothing() || add(true, false).IsNothing())
+          return;
 
         break;
       default:
@@ -1562,6 +1573,25 @@ void CanonicalizeIP(const FunctionCallbackInfo<Value>& args) {
   args.GetReturnValue().Set(val);
 }
 
+void ConvertIpv6StringToBuffer(const FunctionCallbackInfo<Value>& args) {
+  Isolate* isolate = args.GetIsolate();
+  node::Utf8Value ip(isolate, args[0]);
+  unsigned char dst[16];  // IPv6 addresses are 128 bits (16 bytes)
+
+  if (uv_inet_pton(AF_INET6, *ip, dst) != 0) {
+    isolate->ThrowException(Exception::Error(
+        FIXED_ONE_BYTE_STRING(isolate, "Invalid IPv6 address")));
+    return;
+  }
+
+  Local<Object> buffer;
+  if (node::Buffer::Copy(
+          isolate, reinterpret_cast<const char*>(dst), sizeof(dst))
+          .ToLocal(&buffer)) {
+    args.GetReturnValue().Set(buffer);
+  }
+}
+
 void GetAddrInfo(const FunctionCallbackInfo<Value>& args) {
   Environment* env = Environment::GetCurrent(args);
 
@@ -1721,22 +1751,27 @@ void SetServers(const FunctionCallbackInfo<Value>& args) {
   int err;
 
   for (uint32_t i = 0; i < len; i++) {
-    CHECK(arr->Get(env->context(), i).ToLocalChecked()->IsArray());
+    Local<Value> val;
+    if (!arr->Get(env->context(), i).ToLocal(&val)) return;
+    CHECK(val->IsArray());
 
-    Local<Array> elm = arr->Get(env->context(), i).ToLocalChecked().As<Array>();
+    Local<Array> elm = val.As<Array>();
 
-    CHECK(elm->Get(env->context(),
-                   0).ToLocalChecked()->Int32Value(env->context()).FromJust());
-    CHECK(elm->Get(env->context(), 1).ToLocalChecked()->IsString());
-    CHECK(elm->Get(env->context(),
-                   2).ToLocalChecked()->Int32Value(env->context()).FromJust());
+    Local<Value> familyValue;
+    Local<Value> ipValue;
+    Local<Value> portValue;
 
-    int fam = elm->Get(env->context(), 0)
-        .ToLocalChecked()->Int32Value(env->context()).FromJust();
-    node::Utf8Value ip(env->isolate(),
-                       elm->Get(env->context(), 1).ToLocalChecked());
-    int port = elm->Get(env->context(), 2)
-        .ToLocalChecked()->Int32Value(env->context()).FromJust();
+    if (!elm->Get(env->context(), 0).ToLocal(&familyValue)) return;
+    if (!elm->Get(env->context(), 1).ToLocal(&ipValue)) return;
+    if (!elm->Get(env->context(), 2).ToLocal(&portValue)) return;
+
+    CHECK(familyValue->Int32Value(env->context()).FromJust());
+    CHECK(ipValue->IsString());
+    CHECK(portValue->Int32Value(env->context()).FromJust());
+
+    int fam = familyValue->Int32Value(env->context()).FromJust();
+    node::Utf8Value ip(env->isolate(), ipValue);
+    int port = portValue->Int32Value(env->context()).FromJust();
 
     ares_addr_port_node* cur = &servers[i];
 
@@ -1797,7 +1832,7 @@ void SetLocalAddress(const FunctionCallbackInfo<Value>& args) {
   // to 0 (any).
 
   if (uv_inet_pton(AF_INET, *ip0, &addr0) == 0) {
-    ares_set_local_ip4(channel->cares_channel(), ReadUint32BE(addr0));
+    ares_set_local_ip4(channel->cares_channel(), nbytes::ReadUint32BE(addr0));
     type0 = 4;
   } else if (uv_inet_pton(AF_INET6, *ip0, &addr0) == 0) {
     ares_set_local_ip6(channel->cares_channel(), addr0);
@@ -1816,7 +1851,8 @@ void SetLocalAddress(const FunctionCallbackInfo<Value>& args) {
         THROW_ERR_INVALID_ARG_VALUE(env, "Cannot specify two IPv4 addresses.");
         return;
       } else {
-        ares_set_local_ip4(channel->cares_channel(), ReadUint32BE(addr1));
+        ares_set_local_ip4(channel->cares_channel(),
+                           nbytes::ReadUint32BE(addr1));
       }
     } else if (uv_inet_pton(AF_INET6, *ip1, &addr1) == 0) {
       if (type0 == 6) {
@@ -1897,6 +1933,8 @@ void Initialize(Local<Object> target,
   SetMethod(context, target, "getaddrinfo", GetAddrInfo);
   SetMethod(context, target, "getnameinfo", GetNameInfo);
   SetMethodNoSideEffect(context, target, "canonicalizeIP", CanonicalizeIP);
+  SetMethodNoSideEffect(
+      context, target, "convertIpv6StringToBuffer", ConvertIpv6StringToBuffer);
 
   SetMethod(context, target, "strerror", StrError);
 
@@ -1980,6 +2018,7 @@ void RegisterExternalReferences(ExternalReferenceRegistry* registry) {
   registry->Register(GetAddrInfo);
   registry->Register(GetNameInfo);
   registry->Register(CanonicalizeIP);
+  registry->Register(ConvertIpv6StringToBuffer);
   registry->Register(StrError);
   registry->Register(ChannelWrap::New);
 
