@@ -18,6 +18,14 @@
 
 namespace node {
 
+using ncrypto::BignumPointer;
+using ncrypto::DataPointer;
+using ncrypto::ECGroupPointer;
+using ncrypto::ECKeyPointer;
+using ncrypto::ECPointPointer;
+using ncrypto::EVPKeyCtxPointer;
+using ncrypto::EVPKeyPointer;
+using ncrypto::MarkPopErrorOnReturn;
 using v8::Array;
 using v8::ArrayBuffer;
 using v8::BackingStore;
@@ -26,10 +34,11 @@ using v8::FunctionCallbackInfo;
 using v8::FunctionTemplate;
 using v8::Int32;
 using v8::Isolate;
-using v8::Just;
 using v8::JustVoid;
 using v8::Local;
+using v8::LocalVector;
 using v8::Maybe;
+using v8::MaybeLocal;
 using v8::Nothing;
 using v8::Object;
 using v8::String;
@@ -45,22 +54,6 @@ int GetCurveFromName(const char* name) {
   return nid;
 }
 
-int GetOKPCurveFromName(const char* name) {
-  int nid;
-  if (strcmp(name, "Ed25519") == 0) {
-    nid = EVP_PKEY_ED25519;
-  } else if (strcmp(name, "Ed448") == 0) {
-    nid = EVP_PKEY_ED448;
-  } else if (strcmp(name, "X25519") == 0) {
-    nid = EVP_PKEY_X25519;
-  } else if (strcmp(name, "X448") == 0) {
-    nid = EVP_PKEY_X448;
-  } else {
-    nid = NID_undef;
-  }
-  return nid;
-}
-
 void ECDH::Initialize(Environment* env, Local<Object> target) {
   Isolate* isolate = env->isolate();
   Local<Context> context = env->context();
@@ -111,7 +104,7 @@ void ECDH::GetCurves(const FunctionCallbackInfo<Value>& args) {
   std::vector<EC_builtin_curve> curves(num_curves);
   CHECK_EQ(EC_get_builtin_curves(curves.data(), num_curves), num_curves);
 
-  std::vector<Local<Value>> arr(num_curves);
+  LocalVector<Value> arr(env->isolate(), num_curves);
   std::transform(curves.begin(), curves.end(), arr.begin(), [env](auto& curve) {
     return OneByteString(env->isolate(), OBJ_nid2sn(curve.nid));
   });
@@ -119,9 +112,7 @@ void ECDH::GetCurves(const FunctionCallbackInfo<Value>& args) {
 }
 
 ECDH::ECDH(Environment* env, Local<Object> wrap, ECKeyPointer&& key)
-    : BaseObject(env, wrap),
-    key_(std::move(key)),
-    group_(EC_KEY_get0_group(key_.get())) {
+    : BaseObject(env, wrap), key_(std::move(key)), group_(key_.getGroup()) {
   MakeWeak();
   CHECK_NOT_NULL(group_);
 }
@@ -143,7 +134,7 @@ void ECDH::New(const FunctionCallbackInfo<Value>& args) {
   if (nid == NID_undef)
     return THROW_ERR_CRYPTO_INVALID_CURVE(env);
 
-  ECKeyPointer key(EC_KEY_new_by_curve_name(nid));
+  auto key = ECKeyPointer::NewByCurveName(nid);
   if (!key)
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env,
       "Failed to create key using named curve");
@@ -157,35 +148,34 @@ void ECDH::GenerateKeys(const FunctionCallbackInfo<Value>& args) {
   ECDH* ecdh;
   ASSIGN_OR_RETURN_UNWRAP(&ecdh, args.This());
 
-  if (!EC_KEY_generate_key(ecdh->key_.get()))
+  if (!ecdh->key_.generate()) {
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env, "Failed to generate key");
+  }
 }
 
 ECPointPointer ECDH::BufferToPoint(Environment* env,
                                    const EC_GROUP* group,
                                    Local<Value> buf) {
-  int r;
+  ArrayBufferOrViewContents<unsigned char> input(buf);
+  if (!input.CheckSizeInt32()) [[unlikely]] {
+    THROW_ERR_OUT_OF_RANGE(env, "buffer is too big");
+    return {};
+  }
 
-  ECPointPointer pub(EC_POINT_new(group));
+  auto pub = ECPointPointer::New(group);
   if (!pub) {
     THROW_ERR_CRYPTO_OPERATION_FAILED(env,
         "Failed to allocate EC_POINT for a public key");
     return pub;
   }
 
-  ArrayBufferOrViewContents<unsigned char> input(buf);
-  if (UNLIKELY(!input.CheckSizeInt32())) {
-    THROW_ERR_OUT_OF_RANGE(env, "buffer is too big");
-    return ECPointPointer();
+  ncrypto::Buffer<const unsigned char> buffer{
+      .data = input.data(),
+      .len = input.size(),
+  };
+  if (!pub.setFromBuffer(buffer, group)) {
+    return {};
   }
-  r = EC_POINT_oct2point(
-      group,
-      pub.get(),
-      input.data(),
-      input.size(),
-      nullptr);
-  if (!r)
-    return ECPointPointer();
 
   return pub;
 }
@@ -203,10 +193,7 @@ void ECDH::ComputeSecret(const FunctionCallbackInfo<Value>& args) {
   if (!ecdh->IsKeyPairValid())
     return THROW_ERR_CRYPTO_INVALID_KEYPAIR(env);
 
-  ECPointPointer pub(
-      ECDH::BufferToPoint(env,
-                          ecdh->group_,
-                          args[0]));
+  auto pub = ECDH::BufferToPoint(env, ecdh->group_, args[0]);
   if (!pub) {
     args.GetReturnValue().Set(
         FIXED_ONE_BYTE_STRING(env->isolate(),
@@ -224,7 +211,7 @@ void ECDH::ComputeSecret(const FunctionCallbackInfo<Value>& args) {
   }
 
   if (!ECDH_compute_key(
-          bs->Data(), bs->ByteLength(), pub.get(), ecdh->key_.get(), nullptr))
+          bs->Data(), bs->ByteLength(), pub, ecdh->key_.get(), nullptr))
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env, "Failed to compute ECDH key");
 
   Local<ArrayBuffer> ab = ArrayBuffer::New(env->isolate(), std::move(bs));
@@ -242,8 +229,8 @@ void ECDH::GetPublicKey(const FunctionCallbackInfo<Value>& args) {
   ECDH* ecdh;
   ASSIGN_OR_RETURN_UNWRAP(&ecdh, args.This());
 
-  const EC_GROUP* group = EC_KEY_get0_group(ecdh->key_.get());
-  const EC_POINT* pub = EC_KEY_get0_public_key(ecdh->key_.get());
+  const auto group = ecdh->key_.getGroup();
+  const auto pub = ecdh->key_.getPublicKey();
   if (pub == nullptr)
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env,
         "Failed to get ECDH public key");
@@ -265,7 +252,7 @@ void ECDH::GetPrivateKey(const FunctionCallbackInfo<Value>& args) {
   ECDH* ecdh;
   ASSIGN_OR_RETURN_UNWRAP(&ecdh, args.This());
 
-  const BIGNUM* b = EC_KEY_get0_private_key(ecdh->key_.get());
+  auto b = ecdh->key_.getPrivateKey();
   if (b == nullptr)
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env,
         "Failed to get ECDH private key");
@@ -273,10 +260,11 @@ void ECDH::GetPrivateKey(const FunctionCallbackInfo<Value>& args) {
   std::unique_ptr<BackingStore> bs;
   {
     NoArrayBufferZeroFillScope no_zero_fill_scope(env->isolate_data());
-    bs = ArrayBuffer::NewBackingStore(env->isolate(), BN_num_bytes(b));
+    bs = ArrayBuffer::NewBackingStore(env->isolate(),
+                                      BignumPointer::GetByteCount(b));
   }
-  CHECK_EQ(static_cast<int>(bs->ByteLength()),
-           BN_bn2binpad(
+  CHECK_EQ(bs->ByteLength(),
+           BignumPointer::EncodePaddedInto(
                b, static_cast<unsigned char*>(bs->Data()), bs->ByteLength()));
 
   Local<ArrayBuffer> ab = ArrayBuffer::New(env->isolate(), std::move(bs));
@@ -292,11 +280,10 @@ void ECDH::SetPrivateKey(const FunctionCallbackInfo<Value>& args) {
   ASSIGN_OR_RETURN_UNWRAP(&ecdh, args.This());
 
   ArrayBufferOrViewContents<unsigned char> priv_buffer(args[0]);
-  if (UNLIKELY(!priv_buffer.CheckSizeInt32()))
+  if (!priv_buffer.CheckSizeInt32()) [[unlikely]]
     return THROW_ERR_OUT_OF_RANGE(env, "key is too big");
 
-  BignumPointer priv(BN_bin2bn(
-      priv_buffer.data(), priv_buffer.size(), nullptr));
+  BignumPointer priv(priv_buffer.data(), priv_buffer.size());
   if (!priv) {
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env,
         "Failed to convert Buffer to BN");
@@ -307,10 +294,10 @@ void ECDH::SetPrivateKey(const FunctionCallbackInfo<Value>& args) {
         "Private key is not valid for specified curve.");
   }
 
-  ECKeyPointer new_key(EC_KEY_dup(ecdh->key_.get()));
+  auto new_key = ecdh->key_.clone();
   CHECK(new_key);
 
-  int result = EC_KEY_set_private_key(new_key.get(), priv.get());
+  bool result = new_key.setPrivateKey(priv);
   priv.reset();
 
   if (!result) {
@@ -321,24 +308,24 @@ void ECDH::SetPrivateKey(const FunctionCallbackInfo<Value>& args) {
   MarkPopErrorOnReturn mark_pop_error_on_return;
   USE(&mark_pop_error_on_return);
 
-  const BIGNUM* priv_key = EC_KEY_get0_private_key(new_key.get());
+  auto priv_key = new_key.getPrivateKey();
   CHECK_NOT_NULL(priv_key);
 
-  ECPointPointer pub(EC_POINT_new(ecdh->group_));
+  auto pub = ECPointPointer::New(ecdh->group_);
   CHECK(pub);
 
-  if (!EC_POINT_mul(ecdh->group_, pub.get(), priv_key,
-                    nullptr, nullptr, nullptr)) {
+  if (!pub.mul(ecdh->group_, priv_key)) {
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env,
         "Failed to generate ECDH public key");
   }
 
-  if (!EC_KEY_set_public_key(new_key.get(), pub.get()))
+  if (!new_key.setPublicKey(pub)) {
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env,
         "Failed to set generated public key");
+  }
 
   ecdh->key_ = std::move(new_key);
-  ecdh->group_ = EC_KEY_get0_group(ecdh->key_.get());
+  ecdh->group_ = ecdh->key_.getGroup();
 }
 
 void ECDH::SetPublicKey(const FunctionCallbackInfo<Value>& args) {
@@ -351,17 +338,13 @@ void ECDH::SetPublicKey(const FunctionCallbackInfo<Value>& args) {
 
   MarkPopErrorOnReturn mark_pop_error_on_return;
 
-  ECPointPointer pub(
-      ECDH::BufferToPoint(env,
-                          ecdh->group_,
-                          args[0]));
+  auto pub = ECDH::BufferToPoint(env, ecdh->group_, args[0]);
   if (!pub) {
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env,
         "Failed to convert Buffer to EC_POINT");
   }
 
-  int r = EC_KEY_set_public_key(ecdh->key_.get(), pub.get());
-  if (!r) {
+  if (!ecdh->key_.setPublicKey(pub)) {
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env,
         "Failed to set EC_POINT as the public key");
   }
@@ -372,19 +355,18 @@ bool ECDH::IsKeyValidForCurve(const BignumPointer& private_key) {
   CHECK(private_key);
   // Private keys must be in the range [1, n-1].
   // Ref: Section 3.2.1 - http://www.secg.org/sec1-v2.pdf
-  if (BN_cmp(private_key.get(), BN_value_one()) < 0) {
+  if (private_key < BignumPointer::One()) {
     return false;
   }
-  BignumPointer order(BN_new());
+  auto order = BignumPointer::New();
   CHECK(order);
   return EC_GROUP_get_order(group_, order.get(), nullptr) &&
-         BN_cmp(private_key.get(), order.get()) < 0;
+         private_key < order;
 }
 
 bool ECDH::IsKeyPairValid() {
   MarkPopErrorOnReturn mark_pop_error_on_return;
-  USE(&mark_pop_error_on_return);
-  return 1 == EC_KEY_check_key(key_.get());
+  return key_.checkKey();
 }
 
 // Convert the input public key to compressed, uncompressed, or hybrid formats.
@@ -396,7 +378,7 @@ void ECDH::ConvertKey(const FunctionCallbackInfo<Value>& args) {
   CHECK(IsAnyBufferSource(args[0]));
 
   ArrayBufferOrViewContents<char> args0(args[0]);
-  if (UNLIKELY(!args0.CheckSizeInt32()))
+  if (!args0.CheckSizeInt32()) [[unlikely]]
     return THROW_ERR_OUT_OF_RANGE(env, "key is too big");
   if (args0.empty()) return args.GetReturnValue().SetEmptyString();
 
@@ -406,17 +388,12 @@ void ECDH::ConvertKey(const FunctionCallbackInfo<Value>& args) {
   if (nid == NID_undef)
     return THROW_ERR_CRYPTO_INVALID_CURVE(env);
 
-  ECGroupPointer group(
-      EC_GROUP_new_by_curve_name(nid));
-  if (group == nullptr)
+  auto group = ECGroupPointer::NewByCurveName(nid);
+  if (!group)
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env, "Failed to get EC_GROUP");
 
-  ECPointPointer pub(
-      ECDH::BufferToPoint(env,
-                          group.get(),
-                          args[0]));
-
-  if (pub == nullptr) {
+  auto pub = ECDH::BufferToPoint(env, group, args[0]);
+  if (!pub) {
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env,
         "Failed to convert Buffer to EC_POINT");
   }
@@ -427,7 +404,7 @@ void ECDH::ConvertKey(const FunctionCallbackInfo<Value>& args) {
 
   const char* error;
   Local<Object> buf;
-  if (!ECPointToBuffer(env, group.get(), pub.get(), form, &error).ToLocal(&buf))
+  if (!ECPointToBuffer(env, group, pub, form, &error).ToLocal(&buf))
     return THROW_ERR_CRYPTO_OPERATION_FAILED(env, error);
   args.GetReturnValue().Set(buf);
 }
@@ -437,64 +414,53 @@ void ECDHBitsConfig::MemoryInfo(MemoryTracker* tracker) const {
   tracker->TrackField("private", private_);
 }
 
-Maybe<bool> ECDHBitsTraits::EncodeOutput(
-    Environment* env,
-    const ECDHBitsConfig& params,
-    ByteSource* out,
-    v8::Local<v8::Value>* result) {
-  *result = out->ToArrayBuffer(env);
-  return Just(!result->IsEmpty());
+MaybeLocal<Value> ECDHBitsTraits::EncodeOutput(Environment* env,
+                                               const ECDHBitsConfig& params,
+                                               ByteSource* out) {
+  return out->ToArrayBuffer(env);
 }
 
-Maybe<bool> ECDHBitsTraits::AdditionalConfig(
+Maybe<void> ECDHBitsTraits::AdditionalConfig(
     CryptoJobMode mode,
     const FunctionCallbackInfo<Value>& args,
     unsigned int offset,
     ECDHBitsConfig* params) {
   Environment* env = Environment::GetCurrent(args);
 
-  CHECK(args[offset]->IsString());  // curve name
-  CHECK(args[offset + 1]->IsObject());  // public key
-  CHECK(args[offset + 2]->IsObject());  // private key
+  CHECK(args[offset]->IsObject());      // public key
+  CHECK(args[offset + 1]->IsObject());  // private key
 
   KeyObjectHandle* private_key;
   KeyObjectHandle* public_key;
 
-  Utf8Value name(env->isolate(), args[offset]);
+  ASSIGN_OR_RETURN_UNWRAP(&public_key, args[offset], Nothing<void>());
+  ASSIGN_OR_RETURN_UNWRAP(&private_key, args[offset + 1], Nothing<void>());
 
-  ASSIGN_OR_RETURN_UNWRAP(&public_key, args[offset + 1], Nothing<bool>());
-  ASSIGN_OR_RETURN_UNWRAP(&private_key, args[offset + 2], Nothing<bool>());
-
-  if (private_key->Data()->GetKeyType() != kKeyTypePrivate ||
-      public_key->Data()->GetKeyType() != kKeyTypePublic) {
+  if (private_key->Data().GetKeyType() != kKeyTypePrivate ||
+      public_key->Data().GetKeyType() != kKeyTypePublic) {
     THROW_ERR_CRYPTO_INVALID_KEYTYPE(env);
-    return Nothing<bool>();
+    return Nothing<void>();
   }
 
-  params->id_ = GetOKPCurveFromName(*name);
-  params->private_ = private_key->Data();
-  params->public_ = public_key->Data();
+  params->private_ = private_key->Data().addRef();
+  params->public_ = public_key->Data().addRef();
 
-  return Just(true);
+  return JustVoid();
 }
 
 bool ECDHBitsTraits::DeriveBits(Environment* env,
                                 const ECDHBitsConfig& params,
-                                ByteSource* out,
-                                CryptoJobMode mode) {
+                                ByteSource* out) {
   size_t len = 0;
-  ManagedEVPPKey m_privkey = params.private_->GetAsymmetricKey();
-  ManagedEVPPKey m_pubkey = params.public_->GetAsymmetricKey();
+  const auto& m_privkey = params.private_.GetAsymmetricKey();
+  const auto& m_pubkey = params.public_.GetAsymmetricKey();
 
-  switch (params.id_) {
+  switch (m_privkey.id()) {
     case EVP_PKEY_X25519:
       // Fall through
     case EVP_PKEY_X448: {
-      EVPKeyCtxPointer ctx = nullptr;
-      {
-        ctx.reset(EVP_PKEY_CTX_new(m_privkey.get(), nullptr));
-      }
-      Mutex::ScopedLock pub_lock(*m_pubkey.mutex());
+      EVPKeyCtxPointer ctx = m_privkey.newCtx();
+      Mutex::ScopedLock pub_lock(params.public_.mutex());
       if (EVP_PKEY_derive_init(ctx.get()) <= 0 ||
           EVP_PKEY_derive_set_peer(
               ctx.get(),
@@ -516,20 +482,20 @@ bool ECDHBitsTraits::DeriveBits(Environment* env,
     default: {
       const EC_KEY* private_key;
       {
-        Mutex::ScopedLock priv_lock(*m_privkey.mutex());
-        private_key = EVP_PKEY_get0_EC_KEY(m_privkey.get());
+        Mutex::ScopedLock priv_lock(params.private_.mutex());
+        private_key = m_privkey;
       }
 
-      Mutex::ScopedLock pub_lock(*m_pubkey.mutex());
-      const EC_KEY* public_key = EVP_PKEY_get0_EC_KEY(m_pubkey.get());
+      Mutex::ScopedLock pub_lock(params.public_.mutex());
+      const EC_KEY* public_key = m_pubkey;
 
-      const EC_GROUP* group = EC_KEY_get0_group(private_key);
+      const auto group = ECKeyPointer::GetGroup(private_key);
       if (group == nullptr)
         return false;
 
-      CHECK_EQ(EC_KEY_check_key(private_key), 1);
-      CHECK_EQ(EC_KEY_check_key(public_key), 1);
-      const EC_POINT* pub = EC_KEY_get0_public_key(public_key);
+      CHECK(ECKeyPointer::Check(private_key));
+      CHECK(ECKeyPointer::Check(public_key));
+      const auto pub = ECKeyPointer::GetPublicKey(public_key);
       int field_size = EC_GROUP_get_degree(group);
       len = (field_size + 7) / 8;
       ByteSource::Builder buf(len);
@@ -572,7 +538,7 @@ EVPKeyCtxPointer EcKeyGenTraits::Setup(EcKeyPairGenConfig* params) {
         return EVPKeyCtxPointer();
       }
       EVPKeyPointer key_params(raw_params);
-      key_ctx.reset(EVP_PKEY_CTX_new(key_params.get(), nullptr));
+      key_ctx = key_params.newCtx();
     }
   }
 
@@ -592,7 +558,7 @@ EVPKeyCtxPointer EcKeyGenTraits::Setup(EcKeyPairGenConfig* params) {
 //   7. Private Type
 //   8. Cipher
 //   9. Passphrase
-Maybe<bool> EcKeyGenTraits::AdditionalConfig(
+Maybe<void> EcKeyGenTraits::AdditionalConfig(
     CryptoJobMode mode,
     const FunctionCallbackInfo<Value>& args,
     unsigned int* offset,
@@ -605,64 +571,57 @@ Maybe<bool> EcKeyGenTraits::AdditionalConfig(
   params->params.curve_nid = GetCurveFromName(*curve_name);
   if (params->params.curve_nid == NID_undef) {
     THROW_ERR_CRYPTO_INVALID_CURVE(env);
-    return Nothing<bool>();
+    return Nothing<void>();
   }
 
   params->params.param_encoding = args[*offset + 1].As<Int32>()->Value();
   if (params->params.param_encoding != OPENSSL_EC_NAMED_CURVE &&
       params->params.param_encoding != OPENSSL_EC_EXPLICIT_CURVE) {
     THROW_ERR_OUT_OF_RANGE(env, "Invalid param_encoding specified");
-    return Nothing<bool>();
+    return Nothing<void>();
   }
 
   *offset += 2;
 
-  return Just(true);
+  return JustVoid();
 }
 
 namespace {
-WebCryptoKeyExportStatus EC_Raw_Export(
-    KeyObjectData* key_data,
-    const ECKeyExportConfig& params,
-    ByteSource* out) {
-  ManagedEVPPKey m_pkey = key_data->GetAsymmetricKey();
+WebCryptoKeyExportStatus EC_Raw_Export(const KeyObjectData& key_data,
+                                       const ECKeyExportConfig& params,
+                                       ByteSource* out) {
+  const auto& m_pkey = key_data.GetAsymmetricKey();
   CHECK(m_pkey);
-  Mutex::ScopedLock lock(*m_pkey.mutex());
+  Mutex::ScopedLock lock(key_data.mutex());
 
-  const EC_KEY* ec_key = EVP_PKEY_get0_EC_KEY(m_pkey.get());
-
-  size_t len = 0;
+  const EC_KEY* ec_key = m_pkey;
 
   if (ec_key == nullptr) {
-    typedef int (*export_fn)(const EVP_PKEY*, unsigned char*, size_t* len);
-    export_fn fn = nullptr;
-    switch (key_data->GetKeyType()) {
-      case kKeyTypePrivate:
-        fn = EVP_PKEY_get_raw_private_key;
+    switch (key_data.GetKeyType()) {
+      case kKeyTypePrivate: {
+        auto data = m_pkey.rawPrivateKey();
+        if (!data) return WebCryptoKeyExportStatus::INVALID_KEY_TYPE;
+        *out = ByteSource::Allocated(data.release());
         break;
-      case kKeyTypePublic:
-        fn = EVP_PKEY_get_raw_public_key;
+      }
+      case kKeyTypePublic: {
+        auto data = m_pkey.rawPublicKey();
+        if (!data) return WebCryptoKeyExportStatus::INVALID_KEY_TYPE;
+        *out = ByteSource::Allocated(data.release());
         break;
+      }
       case kKeyTypeSecret:
         UNREACHABLE();
     }
-    CHECK_NOT_NULL(fn);
-    // Get the size of the raw key data
-    if (fn(m_pkey.get(), nullptr, &len) == 0)
-      return WebCryptoKeyExportStatus::INVALID_KEY_TYPE;
-    ByteSource::Builder data(len);
-    if (fn(m_pkey.get(), data.data<unsigned char>(), &len) == 0)
-      return WebCryptoKeyExportStatus::INVALID_KEY_TYPE;
-    *out = std::move(data).release(len);
   } else {
-    if (key_data->GetKeyType() != kKeyTypePublic)
+    if (key_data.GetKeyType() != kKeyTypePublic)
       return WebCryptoKeyExportStatus::INVALID_KEY_TYPE;
-    const EC_GROUP* group = EC_KEY_get0_group(ec_key);
-    const EC_POINT* point = EC_KEY_get0_public_key(ec_key);
+    const auto group = ECKeyPointer::GetGroup(ec_key);
+    const auto point = ECKeyPointer::GetPublicKey(ec_key);
     point_conversion_form_t form = POINT_CONVERSION_UNCOMPRESSED;
 
     // Get the allocated data size...
-    len = EC_POINT_point2oct(group, point, form, nullptr, 0, nullptr);
+    size_t len = EC_POINT_point2oct(group, point, form, nullptr, 0, nullptr);
     if (len == 0)
       return WebCryptoKeyExportStatus::FAILED;
     ByteSource::Builder data(len);
@@ -679,44 +638,43 @@ WebCryptoKeyExportStatus EC_Raw_Export(
 }
 }  // namespace
 
-Maybe<bool> ECKeyExportTraits::AdditionalConfig(
+Maybe<void> ECKeyExportTraits::AdditionalConfig(
     const FunctionCallbackInfo<Value>& args,
     unsigned int offset,
     ECKeyExportConfig* params) {
-  return Just(true);
+  return JustVoid();
 }
 
 WebCryptoKeyExportStatus ECKeyExportTraits::DoExport(
-    std::shared_ptr<KeyObjectData> key_data,
+    const KeyObjectData& key_data,
     WebCryptoKeyFormat format,
     const ECKeyExportConfig& params,
     ByteSource* out) {
-  CHECK_NE(key_data->GetKeyType(), kKeyTypeSecret);
+  CHECK_NE(key_data.GetKeyType(), kKeyTypeSecret);
 
   switch (format) {
     case kWebCryptoKeyFormatRaw:
-      return EC_Raw_Export(key_data.get(), params, out);
+      return EC_Raw_Export(key_data, params, out);
     case kWebCryptoKeyFormatPKCS8:
-      if (key_data->GetKeyType() != kKeyTypePrivate)
+      if (key_data.GetKeyType() != kKeyTypePrivate)
         return WebCryptoKeyExportStatus::INVALID_KEY_TYPE;
-      return PKEY_PKCS8_Export(key_data.get(), out);
+      return PKEY_PKCS8_Export(key_data, out);
     case kWebCryptoKeyFormatSPKI: {
-      if (key_data->GetKeyType() != kKeyTypePublic)
+      if (key_data.GetKeyType() != kKeyTypePublic)
         return WebCryptoKeyExportStatus::INVALID_KEY_TYPE;
 
-      ManagedEVPPKey m_pkey = key_data->GetAsymmetricKey();
-      if (EVP_PKEY_id(m_pkey.get()) != EVP_PKEY_EC) {
-        return PKEY_SPKI_Export(key_data.get(), out);
+      const auto& m_pkey = key_data.GetAsymmetricKey();
+      if (m_pkey.id() != EVP_PKEY_EC) {
+        return PKEY_SPKI_Export(key_data, out);
       } else {
         // Ensure exported key is in uncompressed point format.
         // The temporary EC key is so we can have i2d_PUBKEY_bio() write out
         // the header but it is a somewhat silly hoop to jump through because
         // the header is for all practical purposes a static 26 byte sequence
         // where only the second byte changes.
-        Mutex::ScopedLock lock(*m_pkey.mutex());
-        const EC_KEY* ec_key = EVP_PKEY_get0_EC_KEY(m_pkey.get());
-        const EC_GROUP* group = EC_KEY_get0_group(ec_key);
-        const EC_POINT* point = EC_KEY_get0_public_key(ec_key);
+        Mutex::ScopedLock lock(key_data.mutex());
+        const auto group = ECKeyPointer::GetGroup(m_pkey);
+        const auto point = ECKeyPointer::GetPublicKey(m_pkey);
         const point_conversion_form_t form = POINT_CONVERSION_UNCOMPRESSED;
         const size_t need =
             EC_POINT_point2oct(group, point, form, nullptr, 0, nullptr);
@@ -725,22 +683,19 @@ WebCryptoKeyExportStatus ECKeyExportTraits::DoExport(
         const size_t have = EC_POINT_point2oct(
             group, point, form, data.data<unsigned char>(), need, nullptr);
         if (have == 0) return WebCryptoKeyExportStatus::FAILED;
-        ECKeyPointer ec(EC_KEY_new());
-        CHECK_EQ(1, EC_KEY_set_group(ec.get(), group));
-        ECPointPointer uncompressed(EC_POINT_new(group));
-        CHECK_EQ(1,
-                 EC_POINT_oct2point(group,
-                                    uncompressed.get(),
-                                    data.data<unsigned char>(),
-                                    data.size(),
-                                    nullptr));
-        CHECK_EQ(1, EC_KEY_set_public_key(ec.get(), uncompressed.get()));
-        EVPKeyPointer pkey(EVP_PKEY_new());
-        CHECK_EQ(1, EVP_PKEY_set1_EC_KEY(pkey.get(), ec.get()));
-        BIOPointer bio(BIO_new(BIO_s_mem()));
-        CHECK(bio);
-        if (!i2d_PUBKEY_bio(bio.get(), pkey.get()))
-          return WebCryptoKeyExportStatus::FAILED;
+        auto ec = ECKeyPointer::New(group);
+        CHECK(ec);
+        auto uncompressed = ECPointPointer::New(group);
+        ncrypto::Buffer<const unsigned char> buffer{
+            .data = data.data<unsigned char>(),
+            .len = data.size(),
+        };
+        CHECK(uncompressed.setFromBuffer(buffer, group));
+        CHECK(ec.setPublicKey(uncompressed));
+        auto pkey = EVPKeyPointer::New();
+        CHECK(pkey.set(ec));
+        auto bio = pkey.derPublicKey();
+        if (!bio) return WebCryptoKeyExportStatus::FAILED;
         *out = ByteSource::FromBIO(bio);
         return WebCryptoKeyExportStatus::OK;
       }
@@ -750,26 +705,25 @@ WebCryptoKeyExportStatus ECKeyExportTraits::DoExport(
   }
 }
 
-Maybe<void> ExportJWKEcKey(
-    Environment* env,
-    std::shared_ptr<KeyObjectData> key,
-    Local<Object> target) {
-  ManagedEVPPKey m_pkey = key->GetAsymmetricKey();
-  Mutex::ScopedLock lock(*m_pkey.mutex());
-  CHECK_EQ(EVP_PKEY_id(m_pkey.get()), EVP_PKEY_EC);
+Maybe<void> ExportJWKEcKey(Environment* env,
+                           const KeyObjectData& key,
+                           Local<Object> target) {
+  Mutex::ScopedLock lock(key.mutex());
+  const auto& m_pkey = key.GetAsymmetricKey();
+  CHECK_EQ(m_pkey.id(), EVP_PKEY_EC);
 
-  const EC_KEY* ec = EVP_PKEY_get0_EC_KEY(m_pkey.get());
+  const EC_KEY* ec = m_pkey;
   CHECK_NOT_NULL(ec);
 
-  const EC_POINT* pub = EC_KEY_get0_public_key(ec);
-  const EC_GROUP* group = EC_KEY_get0_group(ec);
+  const auto pub = ECKeyPointer::GetPublicKey(ec);
+  const auto group = ECKeyPointer::GetGroup(ec);
 
   int degree_bits = EC_GROUP_get_degree(group);
   int degree_bytes =
     (degree_bits / CHAR_BIT) + (7 + (degree_bits % CHAR_BIT)) / 8;
 
-  BignumPointer x(BN_new());
-  BignumPointer y(BN_new());
+  auto x = BignumPointer::New();
+  auto y = BignumPointer::New();
 
   if (!EC_POINT_get_affine_coordinates(group, pub, x.get(), y.get(), nullptr)) {
     ThrowCryptoError(env, ERR_get_error(),
@@ -803,16 +757,16 @@ Maybe<void> ExportJWKEcKey(
   const int nid = EC_GROUP_get_curve_name(group);
   switch (nid) {
     case NID_X9_62_prime256v1:
-      crv_name = OneByteString(env->isolate(), "P-256");
+      crv_name = FIXED_ONE_BYTE_STRING(env->isolate(), "P-256");
       break;
     case NID_secp256k1:
-      crv_name = OneByteString(env->isolate(), "secp256k1");
+      crv_name = FIXED_ONE_BYTE_STRING(env->isolate(), "secp256k1");
       break;
     case NID_secp384r1:
-      crv_name = OneByteString(env->isolate(), "P-384");
+      crv_name = FIXED_ONE_BYTE_STRING(env->isolate(), "P-384");
       break;
     case NID_secp521r1:
-      crv_name = OneByteString(env->isolate(), "P-521");
+      crv_name = FIXED_ONE_BYTE_STRING(env->isolate(), "P-521");
       break;
     default: {
       THROW_ERR_CRYPTO_JWK_UNSUPPORTED_CURVE(
@@ -827,104 +781,80 @@ Maybe<void> ExportJWKEcKey(
     return Nothing<void>();
   }
 
-  if (key->GetKeyType() == kKeyTypePrivate) {
-    const BIGNUM* pvt = EC_KEY_get0_private_key(ec);
-    return SetEncodedValue(
-      env,
-      target,
-      env->jwk_d_string(),
-      pvt,
-      degree_bytes).IsJust() ? JustVoid() : Nothing<void>();
+  if (key.GetKeyType() == kKeyTypePrivate) {
+    auto pvt = ECKeyPointer::GetPrivateKey(ec);
+    return SetEncodedValue(env, target, env->jwk_d_string(), pvt, degree_bytes);
   }
 
   return JustVoid();
 }
 
 Maybe<void> ExportJWKEdKey(Environment* env,
-                           std::shared_ptr<KeyObjectData> key,
+                           const KeyObjectData& key,
                            Local<Object> target) {
-  ManagedEVPPKey pkey = key->GetAsymmetricKey();
-  Mutex::ScopedLock lock(*pkey.mutex());
+  Mutex::ScopedLock lock(key.mutex());
+  const auto& pkey = key.GetAsymmetricKey();
 
-  const char* curve = nullptr;
-  switch (EVP_PKEY_id(pkey.get())) {
-    case EVP_PKEY_ED25519:
-      curve = "Ed25519";
-      break;
-    case EVP_PKEY_ED448:
-      curve = "Ed448";
-      break;
-    case EVP_PKEY_X25519:
-      curve = "X25519";
-      break;
-    case EVP_PKEY_X448:
-      curve = "X448";
-      break;
-    default:
-      UNREACHABLE();
-  }
-  if (target->Set(
-          env->context(),
-          env->jwk_crv_string(),
-          OneByteString(env->isolate(), curve)).IsNothing()) {
-    return Nothing<void>();
-  }
-
-  size_t len = 0;
-  Local<Value> encoded;
-  Local<Value> error;
-
-  if (!EVP_PKEY_get_raw_public_key(pkey.get(), nullptr, &len))
-    return Nothing<void>();
-
-  ByteSource::Builder out(len);
-
-  if (key->GetKeyType() == kKeyTypePrivate) {
-    if (!EVP_PKEY_get_raw_private_key(
-            pkey.get(), out.data<unsigned char>(), &len) ||
-        !StringBytes::Encode(
-             env->isolate(), out.data<const char>(), len, BASE64URL, &error)
-             .ToLocal(&encoded) ||
-        !target->Set(env->context(), env->jwk_d_string(), encoded).IsJust()) {
-      if (!error.IsEmpty())
-        env->isolate()->ThrowException(error);
-      return Nothing<void>();
+  const char* curve = ([&] {
+    switch (pkey.id()) {
+      case EVP_PKEY_ED25519:
+        return "Ed25519";
+      case EVP_PKEY_ED448:
+        return "Ed448";
+      case EVP_PKEY_X25519:
+        return "X25519";
+      case EVP_PKEY_X448:
+        return "X448";
+      default:
+        UNREACHABLE();
     }
-  }
+  })();
 
-  if (!EVP_PKEY_get_raw_public_key(
-          pkey.get(), out.data<unsigned char>(), &len) ||
-      !StringBytes::Encode(
-           env->isolate(), out.data<const char>(), len, BASE64URL, &error)
-           .ToLocal(&encoded) ||
-      !target->Set(env->context(), env->jwk_x_string(), encoded).IsJust()) {
-    if (!error.IsEmpty())
-      env->isolate()->ThrowException(error);
-    return Nothing<void>();
-  }
+  static constexpr auto trySetKey = [](Environment* env,
+                                       DataPointer data,
+                                       Local<Object> target,
+                                       Local<String> key) {
+    Local<Value> encoded;
+    Local<Value> error;
+    if (!data) return false;
+    const ncrypto::Buffer<const char> out = data;
+    if (!StringBytes::Encode(
+             env->isolate(), out.data, out.len, BASE64URL, &error)
+             .ToLocal(&encoded) ||
+        target->Set(env->context(), key, encoded).IsNothing()) {
+      if (!error.IsEmpty()) env->isolate()->ThrowException(error);
+      return false;
+    }
+    return true;
+  };
 
-  if (target->Set(
-          env->context(),
-          env->jwk_kty_string(),
-          env->jwk_okp_string()).IsNothing()) {
+  if (target
+          ->Set(env->context(),
+                env->jwk_crv_string(),
+                OneByteString(env->isolate(), curve))
+          .IsNothing() ||
+      (key.GetKeyType() == kKeyTypePrivate &&
+       !trySetKey(env, pkey.rawPrivateKey(), target, env->jwk_d_string())) ||
+      !trySetKey(env, pkey.rawPublicKey(), target, env->jwk_x_string()) ||
+      target->Set(env->context(), env->jwk_kty_string(), env->jwk_okp_string())
+          .IsNothing()) {
     return Nothing<void>();
   }
 
   return JustVoid();
 }
 
-std::shared_ptr<KeyObjectData> ImportJWKEcKey(
-    Environment* env,
-    Local<Object> jwk,
-    const FunctionCallbackInfo<Value>& args,
-    unsigned int offset) {
+KeyObjectData ImportJWKEcKey(Environment* env,
+                             Local<Object> jwk,
+                             const FunctionCallbackInfo<Value>& args,
+                             unsigned int offset) {
   CHECK(args[offset]->IsString());  // curve name
   Utf8Value curve(env->isolate(), args[offset].As<String>());
 
   int nid = GetCurveFromName(*curve);
   if (nid == NID_undef) {  // Unknown curve
     THROW_ERR_CRYPTO_INVALID_CURVE(env);
-    return std::shared_ptr<KeyObjectData>();
+    return {};
   }
 
   Local<Value> x_value;
@@ -934,67 +864,68 @@ std::shared_ptr<KeyObjectData> ImportJWKEcKey(
   if (!jwk->Get(env->context(), env->jwk_x_string()).ToLocal(&x_value) ||
       !jwk->Get(env->context(), env->jwk_y_string()).ToLocal(&y_value) ||
       !jwk->Get(env->context(), env->jwk_d_string()).ToLocal(&d_value)) {
-    return std::shared_ptr<KeyObjectData>();
+    return {};
   }
 
   if (!x_value->IsString() ||
       !y_value->IsString() ||
       (!d_value->IsUndefined() && !d_value->IsString())) {
     THROW_ERR_CRYPTO_INVALID_JWK(env, "Invalid JWK EC key");
-    return std::shared_ptr<KeyObjectData>();
+    return {};
   }
 
   KeyType type = d_value->IsString() ? kKeyTypePrivate : kKeyTypePublic;
 
-  ECKeyPointer ec(EC_KEY_new_by_curve_name(nid));
+  auto ec = ECKeyPointer::NewByCurveName(nid);
   if (!ec) {
     THROW_ERR_CRYPTO_INVALID_JWK(env, "Invalid JWK EC key");
-    return std::shared_ptr<KeyObjectData>();
+    return {};
   }
 
   ByteSource x = ByteSource::FromEncodedString(env, x_value.As<String>());
   ByteSource y = ByteSource::FromEncodedString(env, y_value.As<String>());
 
-  if (!EC_KEY_set_public_key_affine_coordinates(
-          ec.get(),
-          x.ToBN().get(),
-          y.ToBN().get())) {
+  if (!ec.setPublicKeyRaw(x.ToBN(), y.ToBN())) {
     THROW_ERR_CRYPTO_INVALID_JWK(env, "Invalid JWK EC key");
-    return std::shared_ptr<KeyObjectData>();
+    return {};
   }
 
   if (type == kKeyTypePrivate) {
     ByteSource d = ByteSource::FromEncodedString(env, d_value.As<String>());
-    if (!EC_KEY_set_private_key(ec.get(), d.ToBN().get())) {
+    if (!ec.setPrivateKey(d.ToBN())) {
       THROW_ERR_CRYPTO_INVALID_JWK(env, "Invalid JWK EC key");
-      return std::shared_ptr<KeyObjectData>();
+      return {};
     }
   }
 
-  EVPKeyPointer pkey(EVP_PKEY_new());
-  CHECK_EQ(EVP_PKEY_set1_EC_KEY(pkey.get(), ec.get()), 1);
+  auto pkey = EVPKeyPointer::New();
+  if (!pkey) return {};
+  CHECK(pkey.set(ec));
 
-  return KeyObjectData::CreateAsymmetric(type, ManagedEVPPKey(std::move(pkey)));
+  return KeyObjectData::CreateAsymmetric(type, std::move(pkey));
 }
 
-Maybe<bool> GetEcKeyDetail(
-    Environment* env,
-    std::shared_ptr<KeyObjectData> key,
-    Local<Object> target) {
-  ManagedEVPPKey m_pkey = key->GetAsymmetricKey();
-  Mutex::ScopedLock lock(*m_pkey.mutex());
-  CHECK_EQ(EVP_PKEY_id(m_pkey.get()), EVP_PKEY_EC);
+Maybe<void> GetEcKeyDetail(Environment* env,
+                           const KeyObjectData& key,
+                           Local<Object> target) {
+  Mutex::ScopedLock lock(key.mutex());
+  const auto& m_pkey = key.GetAsymmetricKey();
+  CHECK_EQ(m_pkey.id(), EVP_PKEY_EC);
 
-  const EC_KEY* ec = EVP_PKEY_get0_EC_KEY(m_pkey.get());
+  const EC_KEY* ec = m_pkey;
   CHECK_NOT_NULL(ec);
 
-  const EC_GROUP* group = EC_KEY_get0_group(ec);
+  const auto group = ECKeyPointer::GetGroup(ec);
   int nid = EC_GROUP_get_curve_name(group);
 
-  return target->Set(
-      env->context(),
-      env->named_curve_string(),
-      OneByteString(env->isolate(), OBJ_nid2sn(nid)));
+  if (target
+          ->Set(env->context(),
+                env->named_curve_string(),
+                OneByteString(env->isolate(), OBJ_nid2sn(nid)))
+          .IsNothing()) {
+    return Nothing<void>();
+  }
+  return JustVoid();
 }
 
 // WebCrypto requires a different format for ECDSA signatures than
@@ -1002,13 +933,12 @@ Maybe<bool> GetEcKeyDetail(
 // implementation here is a adapted from Chromium's impl here:
 // https://github.com/chromium/chromium/blob/7af6cfd/components/webcrypto/algorithms/ecdsa.cc
 
-size_t GroupOrderSize(const ManagedEVPPKey& key) {
-  const EC_KEY* ec = EVP_PKEY_get0_EC_KEY(key.get());
+size_t GroupOrderSize(const EVPKeyPointer& key) {
+  const EC_KEY* ec = key;
   CHECK_NOT_NULL(ec);
-  const EC_GROUP* group = EC_KEY_get0_group(ec);
-  BignumPointer order(BN_new());
-  CHECK(EC_GROUP_get_order(group, order.get(), nullptr));
-  return BN_num_bytes(order.get());
+  auto order = BignumPointer::New();
+  CHECK(EC_GROUP_get_order(ECKeyPointer::GetGroup(ec), order.get(), nullptr));
+  return order.byteLength();
 }
 }  // namespace crypto
 }  // namespace node
