@@ -11,8 +11,9 @@ namespace node {
 
 using v8::FunctionCallbackInfo;
 using v8::Int32;
-using v8::Just;
+using v8::JustVoid;
 using v8::Maybe;
+using v8::MaybeLocal;
 using v8::Nothing;
 using v8::Value;
 
@@ -39,13 +40,10 @@ void PBKDF2Config::MemoryInfo(MemoryTracker* tracker) const {
   }
 }
 
-Maybe<bool> PBKDF2Traits::EncodeOutput(
-    Environment* env,
-    const PBKDF2Config& params,
-    ByteSource* out,
-    v8::Local<v8::Value>* result) {
-  *result = out->ToArrayBuffer(env);
-  return Just(!result->IsEmpty());
+MaybeLocal<Value> PBKDF2Traits::EncodeOutput(Environment* env,
+                                             const PBKDF2Config& params,
+                                             ByteSource* out) {
+  return out->ToArrayBuffer(env);
 }
 
 // The input arguments for the job are:
@@ -55,7 +53,7 @@ Maybe<bool> PBKDF2Traits::EncodeOutput(
 //   4. The number of iterations
 //   5. The number of bytes to generate
 //   6. The digest algorithm name
-Maybe<bool> PBKDF2Traits::AdditionalConfig(
+Maybe<void> PBKDF2Traits::AdditionalConfig(
     CryptoJobMode mode,
     const FunctionCallbackInfo<Value>& args,
     unsigned int offset,
@@ -67,14 +65,14 @@ Maybe<bool> PBKDF2Traits::AdditionalConfig(
   ArrayBufferOrViewContents<char> pass(args[offset]);
   ArrayBufferOrViewContents<char> salt(args[offset + 1]);
 
-  if (UNLIKELY(!pass.CheckSizeInt32())) {
+  if (!pass.CheckSizeInt32()) [[unlikely]] {
     THROW_ERR_OUT_OF_RANGE(env, "pass is too large");
-    return Nothing<bool>();
+    return Nothing<void>();
   }
 
-  if (UNLIKELY(!salt.CheckSizeInt32())) {
+  if (!salt.CheckSizeInt32()) [[unlikely]] {
     THROW_ERR_OUT_OF_RANGE(env, "salt is too large");
-    return Nothing<bool>();
+    return Nothing<void>();
   }
 
   params->pass = mode == kCryptoJobAsync
@@ -92,46 +90,43 @@ Maybe<bool> PBKDF2Traits::AdditionalConfig(
   params->iterations = args[offset + 2].As<Int32>()->Value();
   if (params->iterations < 0) {
     THROW_ERR_OUT_OF_RANGE(env, "iterations must be <= %d", INT_MAX);
-    return Nothing<bool>();
+    return Nothing<void>();
   }
 
   params->length = args[offset + 3].As<Int32>()->Value();
   if (params->length < 0) {
     THROW_ERR_OUT_OF_RANGE(env, "length must be <= %d", INT_MAX);
-    return Nothing<bool>();
+    return Nothing<void>();
   }
 
   Utf8Value name(args.GetIsolate(), args[offset + 4]);
-  params->digest = EVP_get_digestbyname(*name);
+  params->digest = ncrypto::getDigestByName(name.ToStringView());
   if (params->digest == nullptr) {
     THROW_ERR_CRYPTO_INVALID_DIGEST(env, "Invalid digest: %s", *name);
-    return Nothing<bool>();
+    return Nothing<void>();
   }
 
-  return Just(true);
+  return JustVoid();
 }
 
 bool PBKDF2Traits::DeriveBits(Environment* env,
                               const PBKDF2Config& params,
-                              ByteSource* out,
-                              CryptoJobMode mode) {
-  ByteSource::Builder buf(params.length);
-
+                              ByteSource* out) {
   // Both pass and salt may be zero length here.
-  // The generated bytes are stored in buf, which is
-  // assigned to out on success.
+  auto dp = ncrypto::pbkdf2(params.digest,
+                            ncrypto::Buffer<const char>{
+                                .data = params.pass.data<const char>(),
+                                .len = params.pass.size(),
+                            },
+                            ncrypto::Buffer<const unsigned char>{
+                                .data = params.salt.data<unsigned char>(),
+                                .len = params.salt.size(),
+                            },
+                            params.iterations,
+                            params.length);
 
-  if (PKCS5_PBKDF2_HMAC(params.pass.data<char>(),
-                        params.pass.size(),
-                        params.salt.data<unsigned char>(),
-                        params.salt.size(),
-                        params.iterations,
-                        params.digest,
-                        params.length,
-                        buf.data<unsigned char>()) <= 0) {
-    return false;
-  }
-  *out = std::move(buf).release();
+  if (!dp) return false;
+  *out = ByteSource::Allocated(dp.release());
   return true;
 }
 
