@@ -6,14 +6,17 @@ if (process.config.variables.node_without_node_options)
 // Test options specified by env variable.
 
 const assert = require('assert');
+const path = require('path');
 const exec = require('child_process').execFile;
 const { Worker } = require('worker_threads');
 
+const fixtures = require('../common/fixtures');
 const tmpdir = require('../common/tmpdir');
+const { hasOpenSSL3 } = require('../common/crypto');
 tmpdir.refresh();
 
-const printA = require.resolve('../fixtures/printA.js');
-const printSpaceA = require.resolve('../fixtures/print A.js');
+const printA = path.relative(tmpdir.path, fixtures.path('printA.js'));
+const printSpaceA = path.relative(tmpdir.path, fixtures.path('print A.js'));
 
 expectNoWorker(` -r ${printA} `, 'A\nB\n');
 expectNoWorker(`-r ${printA}`, 'A\nB\n');
@@ -26,6 +29,7 @@ expect('--no-deprecation', 'B\n');
 expect('--no-warnings', 'B\n');
 expect('--no_warnings', 'B\n');
 expect('--trace-warnings', 'B\n');
+expect('--no-extra-info-on-fatal-exception', 'B\n');
 expect('--redirect-warnings=_', 'B\n');
 expect('--trace-deprecation', 'B\n');
 expect('--trace-sync-io', 'B\n');
@@ -61,15 +65,18 @@ if (common.isLinux) {
 if (common.hasCrypto) {
   expectNoWorker('--use-openssl-ca', 'B\n');
   expectNoWorker('--use-bundled-ca', 'B\n');
-  expectNoWorker('--openssl-config=_ossl_cfg', 'B\n');
+  if (!hasOpenSSL3)
+    expectNoWorker('--openssl-config=_ossl_cfg', 'B\n');
 }
 
 // V8 options
 expect('--abort_on-uncaught_exception', 'B\n');
 expect('--disallow-code-generation-from-strings', 'B\n');
+expect('--expose-gc', 'B\n');
 expect('--huge-max-old-generation-size', 'B\n');
 expect('--jitless', 'B\n');
 expect('--max-old-space-size=0', 'B\n');
+expect('--max-semi-space-size=0', 'B\n');
 expect('--stack-trace-limit=100',
        /(\s*at f \(\[(eval|worker eval)\]:1:\d*\)\r?\n)/,
        '(function f() { f(); })();',
@@ -78,6 +85,10 @@ expect('--stack-trace-limit=100',
 if (!['arm', 'arm64'].includes(process.arch))
   expect('--interpreted-frames-native-stack', 'B\n');
 
+// Workers can't eval as ES Modules. https://github.com/nodejs/node/issues/30682
+expectNoWorker('--input-type=module',
+               'B\n', 'console.log(await "B")');
+
 function expectNoWorker(opt, want, command, wantsError) {
   expect(opt, want, command, wantsError, false);
 }
@@ -85,7 +96,12 @@ function expectNoWorker(opt, want, command, wantsError) {
 function expect(
   opt, want, command = 'console.log("B")', wantsError = false, testWorker = true
 ) {
-  const argv = ['-e', command];
+  const argv = [
+    // --perf-basic-prof and --perf-basic-prof-only-functions write to /tmp by default.
+    `--perf-basic-prof-path=${tmpdir.path}`,
+    '-e',
+    command,
+  ];
   const opts = {
     cwd: tmpdir.path,
     env: Object.assign({}, process.env, { NODE_OPTIONS: opt }),
